@@ -1,28 +1,38 @@
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import os
-from autogen.agentchat.assistant_agent import ConversableAgent
-from autogen.agentchat.contrib.capabilities.agent_capability import AgentCapability
-from autogen.agentchat.contrib.text_analyzer_agent import TextAnalyzerAgent
-from typing import Dict, Optional, Union, List, Tuple, Any
-import chromadb
-from chromadb.config import Settings
 import pickle
+from typing import Any, Optional, Union
 
-try:
-    from termcolor import colored
-except ImportError:
+from ....formatting_utils import colored
+from ....import_utils import optional_import_block, require_optional_import
+from ...assistant_agent import ConversableAgent
+from ..text_analyzer_agent import TextAnalyzerAgent
+from .agent_capability import AgentCapability
 
-    def colored(x, *args, **kwargs):
-        return x
+with optional_import_block():
+    import chromadb
+    from chromadb.config import Settings
 
 
 class Teachability(AgentCapability):
-    """
-    Teachability uses a vector database to give an agent the ability to remember user teachings,
+    """Teachability uses a vector database to give an agent the ability to remember user teachings,
     where the user is any caller (human or not) sending messages to the teachable agent.
     Teachability is designed to be composable with other agent capabilities.
     To make any conversable agent teachable, instantiate both the agent and the Teachability class,
     then pass the agent to teachability.add_to_agent(agent).
     Note that teachable agents in a group chat must be given unique path_to_db_dir values.
+
+    When adding Teachability to an agent, the following are modified:
+    - The agent's system message is appended with a note about the agent's new ability.
+    - A hook is added to the agent's `process_last_received_message` hookable method,
+    and the hook potentially modifies the last of the received messages to include earlier teachings related to the message.
+    Added teachings do not propagate into the stored message history.
+    If new user teachings are detected, they are added to new memos in the vector database.
     """
 
     def __init__(
@@ -32,17 +42,16 @@ class Teachability(AgentCapability):
         path_to_db_dir: Optional[str] = "./tmp/teachable_agent_db",
         recall_threshold: Optional[float] = 1.5,
         max_num_retrievals: Optional[int] = 10,
-        llm_config: Optional[Union[Dict, bool]] = None,
+        llm_config: Optional[Union[dict[str, Any], bool]] = None,
     ):
-        """
-        Args:
-            verbosity (Optional, int): # 0 (default) for basic info, 1 to add memory operations, 2 for analyzer messages, 3 for memo lists.
-            reset_db (Optional, bool): True to clear the DB before starting. Default False.
-            path_to_db_dir (Optional, str): path to the directory where this particular agent's DB is stored. Default "./tmp/teachable_agent_db"
-            recall_threshold (Optional, float): The maximum distance for retrieved memos, where 0.0 is exact match. Default 1.5. Larger values allow more (but less relevant) memos to be recalled.
-            max_num_retrievals (Optional, int): The maximum number of memos to retrieve from the DB. Default 10.
-            llm_config (dict or False): llm inference configuration passed to TextAnalyzerAgent.
-                If None, TextAnalyzerAgent uses llm_config from the teachable agent.
+        """Args:
+        verbosity (Optional, int): # 0 (default) for basic info, 1 to add memory operations, 2 for analyzer messages, 3 for memo lists.
+        reset_db (Optional, bool): True to clear the DB before starting. Default False.
+        path_to_db_dir (Optional, str): path to the directory where this particular agent's DB is stored. Default "./tmp/teachable_agent_db"
+        recall_threshold (Optional, float): The maximum distance for retrieved memos, where 0.0 is exact match. Default 1.5. Larger values allow more (but less relevant) memos to be recalled.
+        max_num_retrievals (Optional, int): The maximum number of memos to retrieve from the DB. Default 10.
+        llm_config (dict or False): llm inference configuration passed to TextAnalyzerAgent.
+            If None, TextAnalyzerAgent uses llm_config from the teachable agent.
         """
         self.verbosity = verbosity
         self.path_to_db_dir = path_to_db_dir
@@ -82,12 +91,10 @@ class Teachability(AgentCapability):
         """Adds a few arbitrary memos to the DB."""
         self.memo_store.prepopulate()
 
-    def process_last_received_message(self, text):
-        """
-        Appends any relevant memos to the message text, and stores any apparent teachings in new memos.
+    def process_last_received_message(self, text: Union[dict[str, Any], str]):
+        """Appends any relevant memos to the message text, and stores any apparent teachings in new memos.
         Uses TextAnalyzerAgent to make decisions about memo storage and retrieval.
         """
-
         # Try to retrieve relevant memos from the DB.
         expanded_text = text
         if self.memo_store.last_memo_id > 0:
@@ -99,7 +106,7 @@ class Teachability(AgentCapability):
         # Return the (possibly) expanded message text.
         return expanded_text
 
-    def _consider_memo_storage(self, comment):
+    def _consider_memo_storage(self, comment: Union[dict[str, Any], str]):
         """Decides whether to store something from one user comment in the DB."""
         memo_added = False
 
@@ -157,9 +164,8 @@ class Teachability(AgentCapability):
             # Yes. Save them to disk.
             self.memo_store._save_memos()
 
-    def _consider_memo_retrieval(self, comment):
+    def _consider_memo_retrieval(self, comment: Union[dict[str, Any], str]):
         """Decides whether to retrieve memos from the DB, and add them to the chat context."""
-
         # First, use the comment directly as the lookup key.
         if self.verbosity >= 1:
             print(colored("\nLOOK FOR RELEVANT MEMOS, AS QUESTION-ANSWER PAIRS", "light_yellow"))
@@ -191,13 +197,13 @@ class Teachability(AgentCapability):
         # Append the memos to the text of the last message.
         return comment + self._concatenate_memo_texts(memo_list)
 
-    def _retrieve_relevant_memos(self, input_text):
+    def _retrieve_relevant_memos(self, input_text: str) -> list:
         """Returns semantically related memos from the DB."""
         memo_list = self.memo_store.get_related_memos(
             input_text, n_results=self.max_num_retrievals, threshold=self.recall_threshold
         )
 
-        if self.verbosity >= 1:
+        if self.verbosity >= 1:  # noqa: SIM102
             # Was anything retrieved?
             if len(memo_list) == 0:
                 # No. Look at the closest memo.
@@ -209,7 +215,7 @@ class Teachability(AgentCapability):
         memo_list = [memo[1] for memo in memo_list]
         return memo_list
 
-    def _concatenate_memo_texts(self, memo_list):
+    def _concatenate_memo_texts(self, memo_list: list) -> str:
         """Concatenates the memo texts into a single string for inclusion in the chat context."""
         memo_texts = ""
         if len(memo_list) > 0:
@@ -221,7 +227,7 @@ class Teachability(AgentCapability):
             memo_texts = memo_texts + "\n" + info
         return memo_texts
 
-    def _analyze(self, text_to_analyze, analysis_instructions):
+    def _analyze(self, text_to_analyze: Union[dict[str, Any], str], analysis_instructions: Union[dict[str, Any], str]):
         """Asks TextAnalyzerAgent to analyze the given text according to specific instructions."""
         self.analyzer.reset()  # Clear the analyzer's list of messages.
         self.teachable_agent.send(
@@ -233,20 +239,25 @@ class Teachability(AgentCapability):
         return self.teachable_agent.last_message(self.analyzer)["content"]
 
 
+@require_optional_import("chromadb", "teachable")
 class MemoStore:
-    """
-    Provides memory storage and retrieval for a teachable agent, using a vector database.
+    """Provides memory storage and retrieval for a teachable agent, using a vector database.
     Each DB entry (called a memo) is a pair of strings: an input text and an output text.
     The input text might be a question, or a task to perform.
     The output text might be an answer to the question, or advice on how to perform the task.
     Vector embeddings are currently supplied by Chroma's default Sentence Transformers.
     """
 
-    def __init__(self, verbosity, reset, path_to_db_dir):
-        """
-        Args:
-            - verbosity (Optional, int): 1 to print memory operations, 0 to omit them. 3+ to print memo lists.
-            - path_to_db_dir (Optional, str): path to the directory where the DB is stored.
+    def __init__(
+        self,
+        verbosity: Optional[int] = 0,
+        reset: Optional[bool] = False,
+        path_to_db_dir: Optional[str] = "./tmp/teachable_agent_db",
+    ):
+        """Args:
+        - verbosity (Optional, int): 1 to print memory operations, 0 to omit them. 3+ to print memo lists.
+        - reset (Optional, bool): True to clear the DB before starting. Default False.
+        - path_to_db_dir (Optional, str): path to the directory where the DB is stored.
         """
         self.verbosity = verbosity
         self.path_to_db_dir = path_to_db_dir
@@ -264,7 +275,7 @@ class MemoStore:
         self.last_memo_id = 0
         if (not reset) and os.path.exists(self.path_to_dict):
             print(colored("\nLOADING MEMORY FROM DISK", "light_green"))
-            print(colored("    Location = {}".format(self.path_to_dict), "light_green"))
+            print(colored(f"    Location = {self.path_to_dict}", "light_green"))
             with open(self.path_to_dict, "rb") as f:
                 self.uid_text_dict = pickle.load(f)
                 self.last_memo_id = len(self.uid_text_dict)
@@ -282,7 +293,7 @@ class MemoStore:
             input_text, output_text = text
             print(
                 colored(
-                    "  ID: {}\n    INPUT TEXT: {}\n    OUTPUT TEXT: {}".format(uid, input_text, output_text),
+                    f"  ID: {uid}\n    INPUT TEXT: {input_text}\n    OUTPUT TEXT: {output_text}",
                     "light_green",
                 )
             )
@@ -300,7 +311,7 @@ class MemoStore:
         self.uid_text_dict = {}
         self._save_memos()
 
-    def add_input_output_pair(self, input_text, output_text):
+    def add_input_output_pair(self, input_text: str, output_text: str):
         """Adds an input-output pair to the vector DB."""
         self.last_memo_id += 1
         self.vec_db.add(documents=[input_text], ids=[str(self.last_memo_id)])
@@ -308,16 +319,14 @@ class MemoStore:
         if self.verbosity >= 1:
             print(
                 colored(
-                    "\nINPUT-OUTPUT PAIR ADDED TO VECTOR DATABASE:\n  ID\n    {}\n  INPUT\n    {}\n  OUTPUT\n    {}\n".format(
-                        self.last_memo_id, input_text, output_text
-                    ),
+                    f"\nINPUT-OUTPUT PAIR ADDED TO VECTOR DATABASE:\n  ID\n    {self.last_memo_id}\n  INPUT\n    {input_text}\n  OUTPUT\n    {output_text}\n",
                     "light_yellow",
                 )
             )
         if self.verbosity >= 3:
             self.list_memos()
 
-    def get_nearest_memo(self, query_text):
+    def get_nearest_memo(self, query_text: str):
         """Retrieves the nearest memo to the given query text."""
         results = self.vec_db.query(query_texts=[query_text], n_results=1)
         uid, input_text, distance = results["ids"][0][0], results["documents"][0][0], results["distances"][0][0]
@@ -326,15 +335,13 @@ class MemoStore:
         if self.verbosity >= 1:
             print(
                 colored(
-                    "\nINPUT-OUTPUT PAIR RETRIEVED FROM VECTOR DATABASE:\n  INPUT1\n    {}\n  OUTPUT\n    {}\n  DISTANCE\n    {}".format(
-                        input_text, output_text, distance
-                    ),
+                    f"\nINPUT-OUTPUT PAIR RETRIEVED FROM VECTOR DATABASE:\n  INPUT1\n    {input_text}\n  OUTPUT\n    {output_text}\n  DISTANCE\n    {distance}",
                     "light_yellow",
                 )
             )
         return input_text, output_text, distance
 
-    def get_related_memos(self, query_text, n_results, threshold):
+    def get_related_memos(self, query_text: str, n_results: int, threshold: Union[int, float]):
         """Retrieves memos that are related to the given query text within the specified distance threshold."""
         if n_results > len(self.uid_text_dict):
             n_results = len(self.uid_text_dict)
@@ -349,9 +356,7 @@ class MemoStore:
                 if self.verbosity >= 1:
                     print(
                         colored(
-                            "\nINPUT-OUTPUT PAIR RETRIEVED FROM VECTOR DATABASE:\n  INPUT1\n    {}\n  OUTPUT\n    {}\n  DISTANCE\n    {}".format(
-                                input_text, output_text, distance
-                            ),
+                            f"\nINPUT-OUTPUT PAIR RETRIEVED FROM VECTOR DATABASE:\n  INPUT1\n    {input_text}\n  OUTPUT\n    {output_text}\n  DISTANCE\n    {distance}",
                             "light_yellow",
                         )
                     )
@@ -367,24 +372,21 @@ class MemoStore:
         examples.append({"text": "Please verify that each paper you listed actually uses langchain.", "label": "no"})
         examples.append({"text": "Tell gpt the output should still be latex code.", "label": "no"})
         examples.append({"text": "Hint: convert pdfs to text and then answer questions based on them.", "label": "yes"})
-        examples.append(
-            {"text": "To create a good PPT, include enough content to make it interesting.", "label": "yes"}
-        )
-        examples.append(
-            {
-                "text": "No, for this case the columns should be aspects and the rows should be frameworks.",
-                "label": "no",
-            }
-        )
+        examples.append({
+            "text": "To create a good PPT, include enough content to make it interesting.",
+            "label": "yes",
+        })
+        examples.append({
+            "text": "No, for this case the columns should be aspects and the rows should be frameworks.",
+            "label": "no",
+        })
         examples.append({"text": "When writing code, remember to include any libraries that are used.", "label": "yes"})
         examples.append({"text": "Please summarize the papers by Eric Horvitz on bounded rationality.", "label": "no"})
         examples.append({"text": "Compare the h-index of Daniel Weld and Oren Etzioni.", "label": "no"})
-        examples.append(
-            {
-                "text": "Double check to be sure that the columns in a table correspond to what was asked for.",
-                "label": "yes",
-            }
-        )
+        examples.append({
+            "text": "Double check to be sure that the columns in a table correspond to what was asked for.",
+            "label": "yes",
+        })
         for example in examples:
             self.add_input_output_pair(example["text"], example["label"])
         self._save_memos()
