@@ -1,76 +1,82 @@
-from collections import defaultdict
-import openai
-import json
-import time
-import logging
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import copy
+import json
+import logging
+import time
+from collections import defaultdict
+from typing import Any, Optional, Union
 
-from autogen import OpenAIWrapper
-from autogen.oai.openai_utils import retrieve_assistants_by_name
-from autogen.agentchat.agent import Agent
-from autogen.agentchat.assistant_agent import ConversableAgent
-from autogen.agentchat.assistant_agent import AssistantAgent
-from typing import Dict, Optional, Union, List, Tuple, Any
+from ... import OpenAIWrapper
+from ...oai.openai_utils import create_gpt_assistant, retrieve_assistants_by_name, update_gpt_assistant
+from ...runtime_logging import log_new_agent, logging_enabled
+from ..agent import Agent
+from ..assistant_agent import AssistantAgent, ConversableAgent
 
 logger = logging.getLogger(__name__)
 
 
 class GPTAssistantAgent(ConversableAgent):
-    """
-    An experimental AutoGen agent class that leverages the OpenAI Assistant API for conversational capabilities.
+    """An experimental AG2 agent class that leverages the OpenAI Assistant API for conversational capabilities.
     This agent is unique in its reliance on the OpenAI Assistant for state management, differing from other agents like ConversableAgent.
     """
 
+    DEFAULT_MODEL_NAME = "gpt-4-0125-preview"
+
     def __init__(
         self,
         name="GPT Assistant",
         instructions: Optional[str] = None,
-        llm_config: Optional[Union[Dict, bool]] = None,
+        llm_config: Optional[Union[dict[str, Any], bool]] = None,
+        assistant_config: Optional[dict[str, Any]] = None,
         overwrite_instructions: bool = False,
         overwrite_tools: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ):
+        """Args:
+        name (str): name of the agent. It will be used to find the existing assistant by name. Please remember to delete an old assistant with the same name if you intend to create a new assistant with the same name.
+        instructions (str): instructions for the OpenAI assistant configuration.
+        When instructions is not None, the system message of the agent will be
+        set to the provided instructions and used in the assistant run, irrespective
+        of the overwrite_instructions flag. But when instructions is None,
+        and the assistant does not exist, the system message will be set to
+        AssistantAgent.DEFAULT_SYSTEM_MESSAGE. If the assistant exists, the
+        system message will be set to the existing assistant instructions.
+        llm_config (dict or False): llm inference configuration.
+            - model: Model to use for the assistant (gpt-4-1106-preview, gpt-3.5-turbo-1106).
+        assistant_config
+            - assistant_id: ID of the assistant to use. If None, a new assistant will be created.
+            - check_every_ms: check thread run status interval
+            - tools: Give Assistants access to OpenAI-hosted tools like Code Interpreter and Knowledge Retrieval,
+                    or build your own tools using Function calling. ref https://platform.openai.com/docs/assistants/tools
+            - file_ids: (Deprecated) files used by retrieval in run. It is Deprecated, use tool_resources instead. https://platform.openai.com/docs/assistants/migration/what-has-changed.
+            - tool_resources: A set of resources that are used by the assistant's tools. The resources are specific to the type of tool.
+        overwrite_instructions (bool): whether to overwrite the instructions of an existing assistant. This parameter is in effect only when assistant_id is specified in llm_config.
+        overwrite_tools (bool): whether to overwrite the tools of an existing assistant. This parameter is in effect only when assistant_id is specified in llm_config.
+        kwargs (dict): Additional configuration options for the agent.
+            - verbose (bool): If set to True, enables more detailed output from the assistant thread.
+            - Other kwargs: Except verbose, others are passed directly to ConversableAgent.
         """
-        Args:
-            name (str): name of the agent. It will be used to find the existing assistant by name. Please remember to delete an old assistant with the same name if you intend to create a new assistant with the same name.
-            instructions (str): instructions for the OpenAI assistant configuration.
-            When instructions is not None, the system message of the agent will be
-            set to the provided instructions and used in the assistant run, irrespective
-            of the overwrite_instructions flag. But when instructions is None,
-            and the assistant does not exist, the system message will be set to
-            AssistantAgent.DEFAULT_SYSTEM_MESSAGE. If the assistant exists, the
-            system message will be set to the existing assistant instructions.
-            llm_config (dict or False): llm inference configuration.
-                - assistant_id: ID of the assistant to use. If None, a new assistant will be created.
-                - model: Model to use for the assistant (gpt-4-1106-preview, gpt-3.5-turbo-1106).
-                - check_every_ms: check thread run status interval
-                - tools: Give Assistants access to OpenAI-hosted tools like Code Interpreter and Knowledge Retrieval,
-                        or build your own tools using Function calling. ref https://platform.openai.com/docs/assistants/tools
-                - file_ids: files used by retrieval in run
-            overwrite_instructions (bool): whether to overwrite the instructions of an existing assistant. This parameter is in effect only when assistant_id is specified in llm_config.
-            overwrite_tools (bool): whether to overwrite the tools of an existing assistant. This parameter is in effect only when assistant_id is specified in llm_config.
-            kwargs (dict): Additional configuration options for the agent.
-                - verbose (bool): If set to True, enables more detailed output from the assistant thread.
-                - Other kwargs: Except verbose, others are passed directly to ConversableAgent.
-        """
-
         self._verbose = kwargs.pop("verbose", False)
+        openai_client_cfg, openai_assistant_cfg = self._process_assistant_config(llm_config, assistant_config)
+
         super().__init__(
-            name=name, system_message=instructions, human_input_mode="NEVER", llm_config=llm_config, **kwargs
+            name=name, system_message=instructions, human_input_mode="NEVER", llm_config=openai_client_cfg, **kwargs
         )
+        if logging_enabled():
+            log_new_agent(self, locals())
 
-        if llm_config is False:
-            raise ValueError("llm_config=False is not supported for GPTAssistantAgent.")
-
-        # Use AutoGen OpenAIWrapper to create a client
-        model_name = "gpt-4-0125-preview"
-        openai_client_cfg = copy.deepcopy(llm_config)
         # GPTAssistantAgent's azure_deployment param may cause NotFoundError (404) in client.beta.assistants.list()
         # See: https://github.com/microsoft/autogen/pull/1721
+        model_name = self.DEFAULT_MODEL_NAME
         if openai_client_cfg.get("config_list") is not None and len(openai_client_cfg["config_list"]) > 0:
-            model_name = openai_client_cfg["config_list"][0].pop("model", "gpt-4-0125-preview")
+            model_name = openai_client_cfg["config_list"][0].pop("model", self.DEFAULT_MODEL_NAME)
         else:
-            model_name = openai_client_cfg.pop("model", "gpt-4-0125-preview")
+            model_name = openai_client_cfg.pop("model", self.DEFAULT_MODEL_NAME)
 
         logger.warning("OpenAI client config of GPTAssistantAgent(%s) - model: %s", name, model_name)
 
@@ -79,14 +85,16 @@ class GPTAssistantAgent(ConversableAgent):
             logger.warning("GPT Assistant only supports one OpenAI client. Using the first client in the list.")
 
         self._openai_client = oai_wrapper._clients[0]._oai_client
-        openai_assistant_id = llm_config.get("assistant_id", None)
+        openai_assistant_id = openai_assistant_cfg.get("assistant_id", None)
         if openai_assistant_id is None:
             # try to find assistant by name first
             candidate_assistants = retrieve_assistants_by_name(self._openai_client, name)
             if len(candidate_assistants) > 0:
                 # Filter out candidates with the same name but different instructions, file IDs, and function names.
                 candidate_assistants = self.find_matching_assistant(
-                    candidate_assistants, instructions, llm_config.get("tools", []), llm_config.get("file_ids", [])
+                    candidate_assistants,
+                    instructions,
+                    openai_assistant_cfg.get("tools", []),
                 )
 
             if len(candidate_assistants) == 0:
@@ -97,12 +105,12 @@ class GPTAssistantAgent(ConversableAgent):
                         "No instructions were provided for new assistant. Using default instructions from AssistantAgent.DEFAULT_SYSTEM_MESSAGE."
                     )
                     instructions = AssistantAgent.DEFAULT_SYSTEM_MESSAGE
-                self._openai_assistant = self._openai_client.beta.assistants.create(
+                self._openai_assistant = create_gpt_assistant(
+                    self._openai_client,
                     name=name,
                     instructions=instructions,
-                    tools=llm_config.get("tools", []),
                     model=model_name,
-                    file_ids=llm_config.get("file_ids", []),
+                    assistant_config=openai_assistant_cfg,
                 )
             else:
                 logger.warning(
@@ -123,17 +131,20 @@ class GPTAssistantAgent(ConversableAgent):
                 logger.warning(
                     "overwrite_instructions is True. Provided instructions will be used and will modify the assistant in the API"
                 )
-                self._openai_assistant = self._openai_client.beta.assistants.update(
+                self._openai_assistant = update_gpt_assistant(
+                    self._openai_client,
                     assistant_id=openai_assistant_id,
-                    instructions=instructions,
+                    assistant_config={
+                        "instructions": instructions,
+                    },
                 )
             else:
                 logger.warning(
                     "overwrite_instructions is False. Provided instructions will be used without permanently modifying the assistant in the API."
                 )
 
-            # Check if tools are specified in llm_config
-            specified_tools = llm_config.get("tools", None)
+            # Check if tools are specified in assistant_config
+            specified_tools = openai_assistant_cfg.get("tools", None)
 
             if specified_tools is None:
                 # Check if the current assistant has tools defined
@@ -150,29 +161,31 @@ class GPTAssistantAgent(ConversableAgent):
                 logger.warning(
                     "overwrite_tools is True. Provided tools will be used and will modify the assistant in the API"
                 )
-                self._openai_assistant = self._openai_client.beta.assistants.update(
+                self._openai_assistant = update_gpt_assistant(
+                    self._openai_client,
                     assistant_id=openai_assistant_id,
-                    tools=llm_config.get("tools", []),
+                    assistant_config={
+                        "tools": specified_tools,
+                        "tool_resources": openai_assistant_cfg.get("tool_resources", None),
+                    },
                 )
             else:
                 # Tools are specified but overwrite_tools is False; do not update the assistant's tools
                 logger.warning("overwrite_tools is False. Using existing tools from assistant API.")
 
+        self.update_system_message(self._openai_assistant.instructions)
         # lazily create threads
         self._openai_threads = {}
         self._unread_index = defaultdict(int)
-        self.register_reply(Agent, GPTAssistantAgent._invoke_assistant)
-        self.register_reply(Agent, GPTAssistantAgent.check_termination_and_human_reply)
-        self.register_reply(Agent, GPTAssistantAgent.a_check_termination_and_human_reply)
+        self.register_reply([Agent, None], GPTAssistantAgent._invoke_assistant, position=2)
 
     def _invoke_assistant(
         self,
-        messages: Optional[List[Dict]] = None,
+        messages: Optional[list[dict[str, Any]]] = None,
         sender: Optional[Agent] = None,
         config: Optional[Any] = None,
-    ) -> Tuple[bool, Union[str, Dict, None]]:
-        """
-        Invokes the OpenAI assistant to generate a reply based on the given messages.
+    ) -> tuple[bool, Optional[Union[str, dict[str, Any]]]]:
+        """Invokes the OpenAI assistant to generate a reply based on the given messages.
 
         Args:
             messages: A list of messages in the conversation history with the sender.
@@ -182,7 +195,6 @@ class GPTAssistantAgent(ConversableAgent):
         Returns:
             A tuple containing a boolean indicating success and the assistant's reply.
         """
-
         if messages is None:
             messages = self._oai_messages[sender]
         unread_index = self._unread_index[sender] or 0
@@ -196,10 +208,14 @@ class GPTAssistantAgent(ConversableAgent):
         assistant_thread = self._openai_threads[sender]
         # Process each unread message
         for message in pending_messages:
+            if message["content"].strip() == "":
+                continue
+            # Convert message roles to 'user' or 'assistant', by calling _map_role_for_api, to comply with OpenAI API spec
+            api_role = self._map_role_for_api(message["role"])
             self._openai_client.beta.threads.messages.create(
                 thread_id=assistant_thread.id,
                 content=message["content"],
-                role=message["role"],
+                role=api_role,
             )
 
         # Create a new run to get responses from the assistant
@@ -227,9 +243,29 @@ class GPTAssistantAgent(ConversableAgent):
         self._unread_index[sender] = len(self._oai_messages[sender]) + 1
         return True, response
 
-    def _get_run_response(self, thread, run):
+    def _map_role_for_api(self, role: str) -> str:
+        """Maps internal message roles to the roles expected by the OpenAI Assistant API.
+
+        Args:
+            role (str): The role from the internal message.
+
+        Returns:
+            str: The mapped role suitable for the API.
         """
-        Waits for and processes the response of a run from the OpenAI assistant.
+        if role in ["function", "tool"]:
+            return "assistant"
+        elif role == "system":
+            return "system"
+        elif role == "user":
+            return "user"
+        elif role == "assistant":
+            return "assistant"
+        else:
+            # Default to 'assistant' for any other roles not recognized by the API
+            return "assistant"
+
+    def _get_run_response(self, thread, run):
+        """Waits for and processes the response of a run from the OpenAI assistant.
 
         Args:
             run: The run object initiated with the OpenAI assistant.
@@ -247,22 +283,24 @@ class GPTAssistantAgent(ConversableAgent):
                     if msg.run_id == run.id:
                         for content in msg.content:
                             if content.type == "text":
-                                new_messages.append(
-                                    {"role": msg.role, "content": self._format_assistant_message(content.text)}
-                                )
+                                new_messages.append({
+                                    "role": msg.role,
+                                    "content": self._format_assistant_message(content.text),
+                                })
                             elif content.type == "image_file":
-                                new_messages.append(
-                                    {
-                                        "role": msg.role,
-                                        "content": f"Received file id={content.image_file.file_id}",
-                                    }
-                                )
+                                new_messages.append({
+                                    "role": msg.role,
+                                    "content": f"Received file id={content.image_file.file_id}",
+                                })
                 return new_messages
             elif run.status == "requires_action":
                 actions = []
                 for tool_call in run.required_action.submit_tool_outputs.tool_calls:
                     function = tool_call.function
-                    is_exec_success, tool_response = self.execute_function(function.dict(), self._verbose)
+                    tool_call_id = tool_call.id
+                    is_exec_success, tool_response = self.execute_function(
+                        function.dict(), call_id=tool_call_id, verbose=self._verbose
+                    )
                     tool_response["metadata"] = {
                         "tool_call_id": tool_call.id,
                         "run_id": run.id,
@@ -292,8 +330,7 @@ class GPTAssistantAgent(ConversableAgent):
                 raise ValueError(f"Unexpected run status: {run.status}. Full run info:\n\n{run_info})")
 
     def _wait_for_run(self, run_id: str, thread_id: str) -> Any:
-        """
-        Waits for a run to complete or reach a final state.
+        """Waits for a run to complete or reach a final state.
 
         Args:
             run_id: The ID of the run.
@@ -311,10 +348,7 @@ class GPTAssistantAgent(ConversableAgent):
         return run
 
     def _format_assistant_message(self, message_content):
-        """
-        Formats the assistant's message to include annotations and citations.
-        """
-
+        """Formats the assistant's message to include annotations and citations."""
         annotations = message_content.annotations
         citations = []
 
@@ -347,9 +381,7 @@ class GPTAssistantAgent(ConversableAgent):
         return False
 
     def reset(self):
-        """
-        Resets the agent, clearing any existing conversation thread and unread message indices.
-        """
+        """Resets the agent, clearing any existing conversation thread and unread message indices."""
         super().reset()
         for thread in self._openai_threads.values():
             # Delete the existing thread to start fresh in the next conversation
@@ -398,7 +430,7 @@ class GPTAssistantAgent(ConversableAgent):
         print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
 
     @property
-    def oai_threads(self) -> Dict[Agent, Any]:
+    def oai_threads(self) -> dict[Agent, Any]:
         """Return the threads of the agent."""
         return self._openai_threads
 
@@ -411,6 +443,10 @@ class GPTAssistantAgent(ConversableAgent):
     def openai_client(self):
         return self._openai_client
 
+    @property
+    def openai_assistant(self):
+        return self._openai_assistant
+
     def get_assistant_instructions(self):
         """Return the assistant instructions from OAI assistant API"""
         return self._openai_assistant.instructions
@@ -420,22 +456,22 @@ class GPTAssistantAgent(ConversableAgent):
         logger.warning("Permanently deleting assistant...")
         self._openai_client.beta.assistants.delete(self.assistant_id)
 
-    def find_matching_assistant(self, candidate_assistants, instructions, tools, file_ids):
-        """
-        Find the matching assistant from a list of candidate assistants.
-        Filter out candidates with the same name but different instructions, file IDs, and function names.
-        TODO: implement accurate match based on assistant metadata fields.
+    def find_matching_assistant(self, candidate_assistants, instructions, tools):
+        """Find the matching assistant from a list of candidate assistants.
+        Filter out candidates with the same name but different instructions, and function names.
         """
         matching_assistants = []
 
         # Preprocess the required tools for faster comparison
-        required_tool_types = set(tool.get("type") for tool in tools)
-        required_function_names = set(
+        required_tool_types = {
+            "file_search" if tool.get("type") in ["retrieval", "file_search"] else tool.get("type") for tool in tools
+        }
+
+        required_function_names = {
             tool.get("function", {}).get("name")
             for tool in tools
-            if tool.get("type") not in ["code_interpreter", "retrieval"]
-        )
-        required_file_ids = set(file_ids)  # Convert file_ids to a set for unordered comparison
+            if tool.get("type") not in ["code_interpreter", "retrieval", "file_search"]
+        }
 
         for assistant in candidate_assistants:
             # Check if instructions are similar
@@ -448,11 +484,12 @@ class GPTAssistantAgent(ConversableAgent):
                 continue
 
             # Preprocess the assistant's tools
-            assistant_tool_types = set(tool.type for tool in assistant.tools)
-            assistant_function_names = set(tool.function.name for tool in assistant.tools if hasattr(tool, "function"))
-            assistant_file_ids = set(getattr(assistant, "file_ids", []))  # Convert to set for comparison
+            assistant_tool_types = {
+                "file_search" if tool.type in ["retrieval", "file_search"] else tool.type for tool in assistant.tools
+            }
+            assistant_function_names = {tool.function.name for tool in assistant.tools if hasattr(tool, "function")}
 
-            # Check if the tool types, function names, and file IDs match
+            # Check if the tool types, function names match
             if required_tool_types != assistant_tool_types or required_function_names != assistant_function_names:
                 logger.warning(
                     "tools not match, skip assistant(%s): tools %s, functions %s",
@@ -461,11 +498,27 @@ class GPTAssistantAgent(ConversableAgent):
                     assistant_function_names,
                 )
                 continue
-            if required_file_ids != assistant_file_ids:
-                logger.warning("file_ids not match, skip assistant(%s): %s", assistant.id, assistant_file_ids)
-                continue
 
             # Append assistant to matching list if all conditions are met
             matching_assistants.append(assistant)
 
         return matching_assistants
+
+    def _process_assistant_config(self, llm_config, assistant_config):
+        """Process the llm_config and assistant_config to extract the model name and assistant related configurations."""
+        if llm_config is False:
+            raise ValueError("llm_config=False is not supported for GPTAssistantAgent.")
+
+        openai_client_cfg = {} if llm_config is None else copy.deepcopy(llm_config)
+
+        openai_assistant_cfg = {} if assistant_config is None else copy.deepcopy(assistant_config)
+
+        # Move the assistant related configurations to assistant_config
+        # It's important to keep forward compatibility
+        assistant_config_items = ["assistant_id", "tools", "file_ids", "tool_resources", "check_every_ms"]
+        for item in assistant_config_items:
+            if openai_client_cfg.get(item) is not None and openai_assistant_cfg.get(item) is None:
+                openai_assistant_cfg[item] = openai_client_cfg[item]
+            openai_client_cfg.pop(item, None)
+
+        return openai_client_cfg, openai_assistant_cfg
