@@ -1,26 +1,24 @@
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import json
 import logging
-import os
-import pdb
-import re
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Optional
 
-import replicate
 import requests
-from regex import R
 
-from autogen.agentchat.agent import Agent
-from autogen.agentchat.contrib.img_utils import get_image_data, llava_formatter
-from autogen.agentchat.contrib.multimodal_conversable_agent import MultimodalConversableAgent
-from autogen.code_utils import content_str
-
-try:
-    from termcolor import colored
-except ImportError:
-
-    def colored(x, *args, **kwargs):
-        return x
+from ...code_utils import content_str
+from ...formatting_utils import colored
+from ...import_utils import optional_import_block, require_optional_import
+from ..agent import Agent
+from .img_utils import get_image_data, llava_formatter
+from .multimodal_conversable_agent import MultimodalConversableAgent
 
+with optional_import_block():
+    import replicate
 
 logger = logging.getLogger(__name__)
 
@@ -34,17 +32,16 @@ class LLaVAAgent(MultimodalConversableAgent):
     def __init__(
         self,
         name: str,
-        system_message: Optional[Tuple[str, List]] = DEFAULT_LLAVA_SYS_MSG,
+        system_message: Optional[tuple[str, list]] = DEFAULT_LLAVA_SYS_MSG,
         *args,
-        **kwargs,
+        **kwargs: Any,
     ):
-        """
-        Args:
-            name (str): agent name.
-            system_message (str): system message for the ChatCompletion inference.
-                Please override this attribute if you want to reprogram the agent.
-            **kwargs (dict): Please refer to other kwargs in
-                [ConversableAgent](../conversable_agent#__init__).
+        """Args:
+        name (str): agent name.
+        system_message (str): system message for the ChatCompletion inference.
+            Please override this attribute if you want to reprogram the agent.
+        **kwargs (dict): Please refer to other kwargs in
+            [ConversableAgent](/docs/api-reference/autogen/ConversableAgent#conversableagent).
         """
         super().__init__(
             name,
@@ -100,8 +97,14 @@ class LLaVAAgent(MultimodalConversableAgent):
         return True, out
 
 
+@require_optional_import("replicate", "lmm")
 def _llava_call_binary_with_config(
-    prompt: str, images: list, config: dict, max_new_tokens: int = 1000, temperature: float = 0.5, seed: int = 1
+    prompt: str,
+    images: list[Any],
+    config: dict[str, Any],
+    max_new_tokens: int = 1000,
+    temperature: float = 0.5,
+    seed: int = 1,
 ):
     if config["base_url"].find("0.0.0.0") >= 0 or config["base_url"].find("localhost") >= 0:
         llava_mode = "local"
@@ -145,8 +148,14 @@ def _llava_call_binary_with_config(
     return output
 
 
+@require_optional_import("replicate", "lmm")
 def llava_call_binary(
-    prompt: str, images: list, config_list: list, max_new_tokens: int = 1000, temperature: float = 0.5, seed: int = 1
+    prompt: str,
+    images: list[Any],
+    config_list: list[dict[str, Any]],
+    max_new_tokens: int = 1000,
+    temperature: float = 0.5,
+    seed: int = 1,
 ):
     # TODO 1: add caching around the LLaVA call to save compute and cost
     # TODO 2: add `seed` to ensure reproducibility. The seed is not working now.
@@ -160,10 +169,7 @@ def llava_call_binary(
 
 
 def llava_call(prompt: str, llm_config: dict) -> str:
-    """
-    Makes a call to the LLaVA service to generate text based on a given prompt
-    """
-
+    """Makes a call to the LLaVA service to generate text based on a given prompt"""
     prompt, images = llava_formatter(prompt, order_image_tokens=False)
 
     for im in images:
@@ -176,5 +182,5 @@ def llava_call(prompt: str, llm_config: dict) -> str:
         config_list=llm_config["config_list"],
         max_new_tokens=llm_config.get("max_new_tokens", 2000),
         temperature=llm_config.get("temperature", 0.5),
-        seed=llm_config.get("seed", None),
+        seed=llm_config.get("seed"),
     )
