@@ -1,13 +1,23 @@
-import re
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import os
-from pydantic import BaseModel, Extra, root_validator
-from typing import Any, Callable, Dict, List, Optional, Union, Tuple
+import re
 from time import sleep
+from typing import Any, Callable, Dict, Literal, Optional, Union
 
-from autogen.agentchat import Agent, UserProxyAgent
-from autogen.code_utils import UNKNOWN, extract_code, execute_code, infer_lang
-from autogen.math_utils import get_answer
+from pydantic import BaseModel, root_validator
 
+from ...code_utils import UNKNOWN, execute_code, extract_code, infer_lang
+from ...import_utils import optional_import_block, require_optional_import
+from ...math_utils import get_answer
+from .. import Agent, UserProxyAgent
+
+with optional_import_block() as result:
+    import wolframalpha
 
 PROMPTS = {
     # default
@@ -117,7 +127,7 @@ def _add_print_to_last_line(code):
 
 
 def _remove_print(code):
-    """remove all print statements from a string."""
+    """Remove all print statements from a string."""
     lines = code.splitlines()
     lines = [line for line in lines if not line.startswith("print(")]
     return "\n".join(lines)
@@ -133,30 +143,29 @@ class MathUserProxyAgent(UserProxyAgent):
         self,
         name: Optional[str] = "MathChatAgent",  # default set to MathChatAgent
         is_termination_msg: Optional[
-            Callable[[Dict], bool]
+            Callable[[Dict[str, Any]], bool]
         ] = _is_termination_msg_mathchat,  # terminate if \boxed{} in message
-        human_input_mode: Optional[str] = "NEVER",  # Fully automated
-        default_auto_reply: Optional[Union[str, Dict, None]] = DEFAULT_REPLY,
+        human_input_mode: Literal["ALWAYS", "NEVER", "TERMINATE"] = "NEVER",  # Fully automated
+        default_auto_reply: Optional[Union[str, dict[str, Any]]] = DEFAULT_REPLY,
         max_invalid_q_per_step=3,  # a parameter needed in MathChat
-        **kwargs,
+        **kwargs: Any,
     ):
-        """
-        Args:
-            name (str): name of the agent
-            is_termination_msg (function): a function that takes a message in the form of a dictionary and returns a boolean value indicating if this received message is a termination message.
-                The dict can contain the following keys: "content", "role", "name", "function_call".
-            human_input_mode (str): whether to ask for human inputs every time a message is received.
-                Possible values are "ALWAYS", "TERMINATE", "NEVER".
-                (1) When "ALWAYS", the agent prompts for human input every time a message is received.
-                    Under this mode, the conversation stops when the human input is "exit",
-                    or when is_termination_msg is True and there is no human input.
-                (2) When "TERMINATE", the agent only prompts for human input only when a termination message is received or
-                    the number of auto reply reaches the max_consecutive_auto_reply.
-                (3) (Default) When "NEVER", the agent will never prompt for human input. Under this mode, the conversation stops
-                    when the number of auto reply reaches the max_consecutive_auto_reply or when is_termination_msg is True.
-            default_auto_reply (str or dict or None): the default auto reply message when no code execution or llm based reply is generated.
-            max_invalid_q_per_step (int): (ADDED) the maximum number of invalid queries per step.
-            **kwargs (dict): other kwargs in [UserProxyAgent](../user_proxy_agent#__init__).
+        """Args:
+        name (str): name of the agent
+        is_termination_msg (function): a function that takes a message in the form of a dictionary and returns a boolean value indicating if this received message is a termination message.
+            The dict can contain the following keys: "content", "role", "name", "function_call".
+        human_input_mode (str): whether to ask for human inputs every time a message is received.
+            Possible values are "ALWAYS", "TERMINATE", "NEVER".
+            (1) When "ALWAYS", the agent prompts for human input every time a message is received.
+                Under this mode, the conversation stops when the human input is "exit",
+                or when is_termination_msg is True and there is no human input.
+            (2) When "TERMINATE", the agent only prompts for human input only when a termination message is received or
+                the number of auto reply reaches the max_consecutive_auto_reply.
+            (3) (Default) When "NEVER", the agent will never prompt for human input. Under this mode, the conversation stops
+                when the number of auto reply reaches the max_consecutive_auto_reply or when is_termination_msg is True.
+        default_auto_reply (str or dict or None): the default auto reply message when no code execution or llm based reply is generated.
+        max_invalid_q_per_step (int): (ADDED) the maximum number of invalid queries per step.
+        **kwargs (dict): other kwargs in [UserProxyAgent](/docs/api-reference/autogen/UserProxyAgent#userproxyagent).
         """
         super().__init__(
             name=name,
@@ -165,7 +174,7 @@ class MathUserProxyAgent(UserProxyAgent):
             default_auto_reply=default_auto_reply,
             **kwargs,
         )
-        self.register_reply([Agent, None], MathUserProxyAgent._generate_math_reply, 1)
+        self.register_reply([Agent, None], MathUserProxyAgent._generate_math_reply, position=2)
         # fixed var
         self._max_invalid_q_per_step = max_invalid_q_per_step
 
@@ -176,28 +185,35 @@ class MathUserProxyAgent(UserProxyAgent):
         self._previous_code = ""
         self.last_reply = None
 
-    def generate_init_message(self, problem, prompt_type="default", customized_prompt=None):
+    @staticmethod
+    def message_generator(sender, recipient, context):
         """Generate a prompt for the assistant agent with the given problem and prompt.
 
         Args:
-            problem (str): the problem to be solved.
-            prompt_type (str): the type of the prompt. Possible values are "default", "python", "wolfram".
-                (1) "default": the prompt that allows the agent to choose between 3 ways to solve a problem:
-                    1. write a python program to solve it directly.
-                    2. solve it directly without python.
-                    3. solve it step by step with python.
-                (2) "python":
-                    a simplified prompt from the third way of the "default" prompt, that asks the assistant
-                    to solve the problem step by step with python.
-                (3) "two_tools":
-                    a simplified prompt similar to the "python" prompt, but allows the model to choose between
-                    Python and Wolfram Alpha to solve the problem.
-            customized_prompt (str): a customized prompt to be used. If it is not None, the prompt_type will be ignored.
+            sender (Agent): the sender of the message.
+            recipient (Agent): the recipient of the message.
+            context (dict): a dictionary with the following fields:
+                problem (str): the problem to be solved.
+                prompt_type (str, Optional): the type of the prompt. Possible values are "default", "python", "wolfram".
+                    (1) "default": the prompt that allows the agent to choose between 3 ways to solve a problem:
+                        1. write a python program to solve it directly.
+                        2. solve it directly without python.
+                        3. solve it step by step with python.
+                    (2) "python":
+                        a simplified prompt from the third way of the "default" prompt, that asks the assistant
+                        to solve the problem step by step with python.
+                    (3) "two_tools":
+                        a simplified prompt similar to the "python" prompt, but allows the model to choose between
+                        Python and Wolfram Alpha to solve the problem.
+                customized_prompt (str, Optional): a customized prompt to be used. If it is not None, the prompt_type will be ignored.
 
         Returns:
             str: the generated prompt ready to be sent to the assistant agent.
         """
-        self._reset()
+        sender._reset()
+        problem = context.get("problem")
+        prompt_type = context.get("prompt_type", "default")
+        customized_prompt = context.get("customized_prompt", None)
         if customized_prompt is not None:
             return customized_prompt + problem
         return PROMPTS[prompt_type] + problem
@@ -278,7 +294,7 @@ class MathUserProxyAgent(UserProxyAgent):
 
     def _generate_math_reply(
         self,
-        messages: Optional[List[Dict]] = None,
+        messages: Optional[list[dict]] = None,
         sender: Optional[Agent] = None,
         config: Optional[Any] = None,
     ):
@@ -350,11 +366,11 @@ class MathUserProxyAgent(UserProxyAgent):
 # THE SOFTWARE.
 
 
-def get_from_dict_or_env(data: Dict[str, Any], key: str, env_key: str, default: Optional[str] = None) -> str:
+def get_from_dict_or_env(data: dict[str, Any], key: str, env_key: str, default: Optional[str] = None) -> str:
     """Get a value from a dictionary or an environment variable."""
-    if key in data and data[key]:
+    if data.get(key):
         return data[key]
-    elif env_key in os.environ and os.environ[env_key]:
+    elif os.environ.get(env_key):
         return os.environ[env_key]
     elif default is not None:
         return default
@@ -381,28 +397,20 @@ class WolframAlphaAPIWrapper(BaseModel):
     wolfram_client: Any  #: :meta private:
     wolfram_alpha_appid: Optional[str] = None
 
-    class Config:
-        """Configuration for this pydantic object."""
-
-        extra = Extra.forbid
-
     @root_validator(skip_on_failure=True)
-    def validate_environment(cls, values: Dict) -> Dict:
+    @classmethod
+    @require_optional_import("wolframalpha", "mathchat")
+    def validate_environment(cls, values: dict) -> dict:
         """Validate that api key and python package exists in environment."""
         wolfram_alpha_appid = get_from_dict_or_env(values, "wolfram_alpha_appid", "WOLFRAM_ALPHA_APPID")
         values["wolfram_alpha_appid"] = wolfram_alpha_appid
 
-        try:
-            import wolframalpha
-
-        except ImportError:
-            raise ImportError("wolframalpha is not installed. " "Please install it with `pip install wolframalpha`")
         client = wolframalpha.Client(wolfram_alpha_appid)
         values["wolfram_client"] = client
 
         return values
 
-    def run(self, query: str) -> Tuple[str, bool]:
+    def run(self, query: str) -> tuple[str, bool]:
         """Run query through WolframAlpha and parse result."""
         from urllib.error import HTTPError
 
