@@ -1,27 +1,20 @@
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import copy
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Optional, Union
 
-from autogen import OpenAIWrapper
-from autogen.agentchat import Agent, ConversableAgent
-from autogen.agentchat.contrib.img_utils import (
-    convert_base64_to_data_uri,
+from ... import OpenAIWrapper
+from ...code_utils import content_str
+from .. import Agent, ConversableAgent
+from ..contrib.img_utils import (
     gpt4v_formatter,
     message_formatter_pil_to_b64,
-    pil_to_data_uri,
 )
 
-from ..._pydantic import model_dump
-
-try:
-    from termcolor import colored
-except ImportError:
-
-    def colored(x, *args, **kwargs):
-        return x
-
-
-from autogen.code_utils import content_str
-
 DEFAULT_LMM_SYS_MSG = """You are a helpful AI assistant."""
 DEFAULT_MODEL = "gpt-4-vision-preview"
 
@@ -34,18 +27,17 @@ class MultimodalConversableAgent(ConversableAgent):
     def __init__(
         self,
         name: str,
-        system_message: Optional[Union[str, List]] = DEFAULT_LMM_SYS_MSG,
+        system_message: Optional[Union[str, list]] = DEFAULT_LMM_SYS_MSG,
         is_termination_msg: str = None,
         *args,
-        **kwargs,
+        **kwargs: Any,
     ):
-        """
-        Args:
-            name (str): agent name.
-            system_message (str): system message for the OpenAIWrapper inference.
-                Please override this attribute if you want to reprogram the agent.
-            **kwargs (dict): Please refer to other kwargs in
-                [ConversableAgent](../conversable_agent#__init__).
+        """Args:
+        name (str): agent name.
+        system_message (str): system message for the OpenAIWrapper inference.
+            Please override this attribute if you want to reprogram the agent.
+        **kwargs (dict): Please refer to other kwargs in
+            [ConversableAgent](/docs/api-reference/autogen/ConversableAgent#conversableagent).
         """
         super().__init__(
             name,
@@ -63,21 +55,13 @@ class MultimodalConversableAgent(ConversableAgent):
         )
 
         # Override the `generate_oai_reply`
-        def _replace_reply_func(arr, x, y):
-            for item in arr:
-                if item["reply_func"] is x:
-                    item["reply_func"] = y
-
-        _replace_reply_func(
-            self._reply_func_list, ConversableAgent.generate_oai_reply, MultimodalConversableAgent.generate_oai_reply
-        )
-        _replace_reply_func(
-            self._reply_func_list,
+        self.replace_reply_func(ConversableAgent.generate_oai_reply, MultimodalConversableAgent.generate_oai_reply)
+        self.replace_reply_func(
             ConversableAgent.a_generate_oai_reply,
             MultimodalConversableAgent.a_generate_oai_reply,
         )
 
-    def update_system_message(self, system_message: Union[Dict, List, str]):
+    def update_system_message(self, system_message: Union[dict[str, Any], list[str], str]):
         """Update the system message.
 
         Args:
@@ -87,7 +71,7 @@ class MultimodalConversableAgent(ConversableAgent):
         self._oai_system_message[0]["role"] = "system"
 
     @staticmethod
-    def _message_to_dict(message: Union[Dict, List, str]) -> Dict:
+    def _message_to_dict(message: Union[dict[str, Any], list[str], str]) -> dict:
         """Convert a message to a dictionary. This implementation
         handles the GPT-4V formatting for easier prompts.
 
@@ -116,10 +100,10 @@ class MultimodalConversableAgent(ConversableAgent):
 
     def generate_oai_reply(
         self,
-        messages: Optional[List[Dict]] = None,
+        messages: Optional[list[dict[str, Any]]] = None,
         sender: Optional[Agent] = None,
         config: Optional[OpenAIWrapper] = None,
-    ) -> Tuple[bool, Union[str, Dict, None]]:
+    ) -> tuple[bool, Optional[Union[str, dict[str, Any]]]]:
         """Generate a reply using autogen.oai."""
         client = self.client if config is None else config
         if client is None:
@@ -130,10 +114,12 @@ class MultimodalConversableAgent(ConversableAgent):
         messages_with_b64_img = message_formatter_pil_to_b64(self._oai_system_message + messages)
 
         # TODO: #1143 handle token limit exceeded error
-        response = client.create(context=messages[-1].pop("context", None), messages=messages_with_b64_img)
+        response = client.create(
+            context=messages[-1].pop("context", None), messages=messages_with_b64_img, agent=self.name
+        )
 
         # TODO: line 301, line 271 is converting messages to dict. Can be removed after ChatCompletionMessage_to_dict is merged.
         extracted_response = client.extract_text_or_completion_object(response)[0]
         if not isinstance(extracted_response, str):
-            extracted_response = model_dump(extracted_response)
+            extracted_response = extracted_response.model_dump()
         return True, extracted_response
