@@ -1,3 +1,9 @@
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
 import logging
 import os
 import pathlib
@@ -6,14 +12,17 @@ import string
 import subprocess
 import sys
 import time
+import venv
 from concurrent.futures import ThreadPoolExecutor, TimeoutError
 from hashlib import md5
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
-
-from autogen import oai
+from types import SimpleNamespace
+from typing import Any, Callable, Optional, Union
 
 import docker
 
+from . import oai
+from .types import UserMessageImageContentPart, UserMessageTextContentPart
+
 SENTINEL = object()
 DEFAULT_MODEL = "gpt-4"
 FAST_MODEL = "gpt-3.5-turbo"
@@ -32,13 +41,14 @@ UNKNOWN = "unknown"
 TIMEOUT_MSG = "Timeout"
 DEFAULT_TIMEOUT = 600
 WIN32 = sys.platform == "win32"
-PATH_SEPARATOR = WIN32 and "\\" or "/"
+PATH_SEPARATOR = (WIN32 and "\\") or "/"
+PYTHON_VARIANTS = ["python", "Python", "py"]
 
 logger = logging.getLogger(__name__)
 
 
-def content_str(content: Union[str, List[Dict[str, Any]], None]) -> str:
-    """Converts `content` into a string format.
+def content_str(content: Union[str, list[Union[UserMessageTextContentPart, UserMessageImageContentPart]], None]) -> str:
+    """Converts the `content` field of an OpenAI message into a string format.
 
     This function processes content that may be a string, a list of mixed text and image URLs, or None,
     and converts it into a string. Text is directly appended to the result string, while image URLs are
@@ -79,7 +89,7 @@ def content_str(content: Union[str, List[Dict[str, Any]], None]) -> str:
 
 
 def infer_lang(code: str) -> str:
-    """infer the language for the code.
+    """Infer the language for the code.
     TODO: make it robust.
     """
     if code.startswith("python ") or code.startswith("pip") or code.startswith("python3 "):
@@ -97,8 +107,8 @@ def infer_lang(code: str) -> str:
 # TODO: In the future move, to better support https://spec.commonmark.org/0.30/#fenced-code-blocks
 #       perhaps by using a full Markdown parser.
 def extract_code(
-    text: Union[str, List], pattern: str = CODE_BLOCK_PATTERN, detect_single_line_code: bool = False
-) -> List[Tuple[str, str]]:
+    text: Union[str, list], pattern: str = CODE_BLOCK_PATTERN, detect_single_line_code: bool = False
+) -> list[tuple[str, str]]:
     """Extract code from a text.
 
     Args:
@@ -135,8 +145,8 @@ def extract_code(
     return extracted
 
 
-def generate_code(pattern: str = CODE_BLOCK_PATTERN, **config) -> Tuple[str, float]:
-    """(openai<1) Generate code.
+def generate_code(pattern: str = CODE_BLOCK_PATTERN, **config) -> tuple[str, float]:
+    """`(openai<1)` Generate code.
 
     Args:
         pattern (Optional, str): The regular expression pattern for finding the code block.
@@ -161,10 +171,10 @@ The current implementation of the function is as follows:
 
 
 def improve_function(file_name, func_name, objective, **config):
-    """(openai<1) Improve the function to achieve the objective."""
+    """`(openai<1)` Improve the function to achieve the objective."""
     params = {**_IMPROVE_FUNCTION_CONFIG, **config}
     # read the entire file into a str
-    with open(file_name, "r") as f:
+    with open(file_name) as f:
         file_string = f.read()
     response = oai.Completion.create(
         {"func_name": func_name, "objective": objective, "file_string": file_string}, **params
@@ -182,7 +192,7 @@ _IMPROVE_CODE_CONFIG = {
 
 
 def improve_code(files, objective, suggest_only=True, **config):
-    """(openai<1) Improve the code to achieve a given objective.
+    """`(openai<1)` Improve the code to achieve a given objective.
 
     Args:
         files (list): A list of file names containing the source code.
@@ -197,7 +207,7 @@ def improve_code(files, objective, suggest_only=True, **config):
     code = ""
     for file_name in files:
         # read the entire file into a string
-        with open(file_name, "r") as f:
+        with open(file_name) as f:
             file_string = f.read()
         code += f"""{file_name}:
 {file_string}
@@ -218,8 +228,7 @@ def get_powershell_command():
         result = subprocess.run(["powershell", "$PSVersionTable.PSVersion.Major"], capture_output=True, text=True)
         if result.returncode == 0:
             return "powershell"
-
-    except FileNotFoundError:
+    except (FileNotFoundError, NotADirectoryError):
         # This means that 'powershell' command is not found so now we try looking for 'pwsh'
         try:
             result = subprocess.run(
@@ -227,22 +236,32 @@ def get_powershell_command():
             )
             if result.returncode == 0:
                 return "pwsh"
-
-        except FileNotFoundError:
-            if WIN32:
-                logging.warning("Neither powershell nor pwsh is installed but it is a Windows OS")
-            return None
+        except FileExistsError as e:
+            raise FileNotFoundError(
+                "Neither powershell.exe nor pwsh.exe is present in the system. "
+                "Please install PowerShell and try again. "
+            ) from e
+        except NotADirectoryError as e:
+            raise NotADirectoryError(
+                "PowerShell is either not installed or its path is not given "
+                "properly in the environment variable PATH. Please check the "
+                "path and try again. "
+            ) from e
+    except PermissionError as e:
+        raise PermissionError("No permission to run powershell.") from e
 
 
-powershell_command = get_powershell_command()
-
-
-def _cmd(lang):
-    if lang.startswith("python") or lang in ["bash", "sh", powershell_command]:
+def _cmd(lang: str) -> str:
+    if lang in PYTHON_VARIANTS:
+        return "python"
+    if lang.startswith("python") or lang in ["bash", "sh"]:
         return lang
     if lang in ["shell"]:
         return "sh"
+    if lang == "javascript":
+        return "node"
     if lang in ["ps1", "pwsh", "powershell"]:
+        powershell_command = get_powershell_command()
         return powershell_command
 
     raise NotImplementedError(f"{lang} not recognized in code execution")
@@ -271,7 +290,7 @@ def in_docker_container() -> bool:
     return os.path.exists("/.dockerenv")
 
 
-def decide_use_docker(use_docker) -> bool:
+def decide_use_docker(use_docker: Optional[bool]) -> Optional[bool]:
     if use_docker is None:
         env_var_use_docker = os.environ.get("AUTOGEN_USE_DOCKER", "True")
 
@@ -338,9 +357,9 @@ def execute_code(
     timeout: Optional[int] = None,
     filename: Optional[str] = None,
     work_dir: Optional[str] = None,
-    use_docker: Union[List[str], str, bool] = SENTINEL,
+    use_docker: Union[list[str], str, bool] = SENTINEL,
     lang: Optional[str] = "python",
-) -> Tuple[int, str, Optional[str]]:
+) -> tuple[int, str, Optional[str]]:
     """Execute code in a docker container.
     This function is not tested on MacOS.
 
@@ -534,8 +553,8 @@ assertions:""",
 }
 
 
-def generate_assertions(definition: str, **config) -> Tuple[str, float]:
-    """(openai<1) Generate assertions for a function.
+def generate_assertions(definition: str, **config) -> tuple[str, float]:
+    """`(openai<1)` Generate assertions for a function.
 
     Args:
         definition (str): The function definition, including the signature and docstr.
@@ -564,15 +583,15 @@ def _remove_check(response):
 
 
 def eval_function_completions(
-    responses: List[str],
+    responses: list[str],
     definition: str,
     test: Optional[str] = None,
     entry_point: Optional[str] = None,
-    assertions: Optional[Union[str, Callable[[str], Tuple[str, float]]]] = None,
+    assertions: Optional[Union[str, Callable[[str], tuple[str, float]]]] = None,
     timeout: Optional[float] = 3,
     use_docker: Optional[bool] = True,
-) -> Dict:
-    """(openai<1) Select a response from a list of responses for the function completion task (using generated assertions), and/or evaluate if the task is successful using a gold test.
+) -> dict:
+    """`(openai<1)` Select a response from a list of responses for the function completion task (using generated assertions), and/or evaluate if the task is successful using a gold test.
 
     Args:
         responses (list): The list of responses.
@@ -662,7 +681,7 @@ class PassAssertionFilter:
         self.metrics = self.responses = None
 
     def pass_assertions(self, context, response, **_):
-        """(openai<1) Check if the response passes the assertions."""
+        """`(openai<1)` Check if the response passes the assertions."""
         responses = oai.Completion.extract_text(response)
         metrics = eval_function_completions(responses, context["definition"], assertions=self._assertions)
         self._assertions = metrics["assertions"]
@@ -674,10 +693,10 @@ class PassAssertionFilter:
 
 def implement(
     definition: str,
-    configs: Optional[List[Dict]] = None,
-    assertions: Optional[Union[str, Callable[[str], Tuple[str, float]]]] = generate_assertions,
-) -> Tuple[str, float]:
-    """(openai<1) Implement a function from a definition.
+    configs: Optional[list[dict[str, Any]]] = None,
+    assertions: Optional[Union[str, Callable[[str], tuple[str, float]]]] = generate_assertions,
+) -> tuple[str, float]:
+    """`(openai<1)` Implement a function from a definition.
 
     Args:
         definition (str): The function definition, including the signature and docstr.
@@ -709,3 +728,20 @@ def implement(
     #     cost += metrics["gen_cost"]
     #     if metrics["succeed_assertions"] or i == len(configs) - 1:
     #         return responses[metrics["index_selected"]], cost, i
+
+
+def create_virtual_env(dir_path: str, **env_args) -> SimpleNamespace:
+    """Creates a python virtual environment and returns the context.
+
+    Args:
+        dir_path (str): Directory path where the env will be created.
+        **env_args: Any extra args to pass to the `EnvBuilder`
+
+    Returns:
+        SimpleNamespace: the virtual env context object.
+    """
+    if not env_args:
+        env_args = {"with_pip": True}
+    env_builder = venv.EnvBuilder(**env_args)
+    env_builder.create(dir_path)
+    return env_builder.ensure_directories(dir_path)
