@@ -1,33 +1,63 @@
-from time import sleep
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
+
 import logging
-import time
-from typing import List, Optional, Dict, Callable, Union
-import sys
 import shutil
-import numpy as np
-from flaml import tune, BlendSearch
-from flaml.tune.space import is_constant
-from flaml.automl.logger import logger_formatter
-from .openai_utils import get_key
+import sys
+import time
 from collections import defaultdict
+from time import sleep
+from typing import Any, Callable, Optional, Union
+
+from ..doc_utils import export_module
+
+# Adding a NullHandler to silence FLAML log warning during
+# import
+flaml_logger = logging.getLogger("flaml")
+null_handler = logging.NullHandler()
+flaml_logger.addHandler(null_handler)
+
+from ..import_utils import optional_import_block, require_optional_import
+
+with optional_import_block() as result:
+    import numpy as np
+    from flaml import BlendSearch, tune
+    from flaml.tune.space import is_constant
+
+FLAML_INSTALLED = result.is_successful
+
+# Restore logging by removing the NullHandler
+flaml_logger.removeHandler(null_handler)
+
+from ..import_utils import optional_import_block
+from .client_utils import logging_formatter
+from .openai_utils import get_key
 
 try:
-    import openai
-    from openai import (
-        RateLimitError,
-        APIError,
-        BadRequestError,
-        APIConnectionError,
-        Timeout,
-        AuthenticationError,
-    )
-    from openai import Completion as openai_Completion
-    import diskcache
+    with optional_import_block() as result:
+        import diskcache
+        import openai
+        from openai import (
+            APIConnectionError,
+            APIError,
+            AuthenticationError,
+            BadRequestError,
+            RateLimitError,
+            Timeout,
+        )
+        from openai import Completion as OpenAICompletion
 
-    ERROR = None
-    assert openai.__version__ < "1"
+    if result.is_successful:
+        ERROR = None
+        assert openai.__version__ < "1"
+    else:
+        raise ImportError("openai<1 is required.")
 except (AssertionError, ImportError):
-    openai_Completion = object
+    OpenAICompletion = object
     # The autogen.Completion class requires openai<1
     ERROR = AssertionError("(Deprecated) The autogen.Completion class requires openai<1 and diskcache. ")
 
@@ -35,12 +65,13 @@ logger = logging.getLogger(__name__)
 if not logger.handlers:
     # Add the console handler.
     _ch = logging.StreamHandler(stream=sys.stdout)
-    _ch.setFormatter(logger_formatter)
+    _ch.setFormatter(logging_formatter)
     logger.addHandler(_ch)
 
 
-class Completion(openai_Completion):
-    """(openai<1) A class for OpenAI completion API.
+@export_module("autogen")
+class Completion(OpenAICompletion):
+    """`(openai<1)` A class for OpenAI completion API.
 
     It also supports: ChatCompletion, Azure OpenAI API.
     """
@@ -63,7 +94,7 @@ class Completion(openai_Completion):
     }
 
     # price per 1k tokens
-    price1K = {
+    price1K = {  # noqa: N815
         "text-ada-001": 0.0004,
         "text-babbage-001": 0.0005,
         "text-curie-001": 0.002,
@@ -88,26 +119,26 @@ class Completion(openai_Completion):
         "gpt-4-32k-0613": (0.06, 0.12),
     }
 
-    default_search_space = {
-        "model": tune.choice(
-            [
+    default_search_space = (
+        {
+            "model": tune.choice([
                 "text-ada-001",
                 "text-babbage-001",
                 "text-davinci-003",
                 "gpt-3.5-turbo",
                 "gpt-4",
-            ]
-        ),
-        "temperature_or_top_p": tune.choice(
-            [
+            ]),
+            "temperature_or_top_p": tune.choice([
                 {"temperature": tune.uniform(0, 2)},
                 {"top_p": tune.uniform(0, 1)},
-            ]
-        ),
-        "max_tokens": tune.lograndint(50, 1000),
-        "n": tune.randint(1, 100),
-        "prompt": "{prompt}",
-    }
+            ]),
+            "max_tokens": tune.lograndint(50, 1000),
+            "n": tune.randint(1, 100),
+            "prompt": "{prompt}",
+        }
+        if FLAML_INSTALLED
+        else {}
+    )
 
     cache_seed = 41
     cache_path = f".cache/{cache_seed}"
@@ -154,7 +185,7 @@ class Completion(openai_Completion):
             cache.clear()
 
     @classmethod
-    def _book_keeping(cls, config: Dict, response):
+    def _book_keeping(cls, config: dict[str, Any], response) -> None:
         """Book keeping for the created completions."""
         if response != -1 and "cost" not in response:
             response["cost"] = cls.cost(response)
@@ -176,14 +207,12 @@ class Completion(openai_Completion):
                 key = get_key([config["prompt"]] + [choice.get("text") for choice in response["choices"]])
             value["created_at"].append(cls._count_create)
             value["cost"].append(response["cost"])
-            value["token_count"].append(
-                {
-                    "model": response["model"],
-                    "prompt_tokens": response["usage"]["prompt_tokens"],
-                    "completion_tokens": response["usage"].get("completion_tokens", 0),
-                    "total_tokens": response["usage"]["total_tokens"],
-                }
-            )
+            value["token_count"].append({
+                "model": response["model"],
+                "prompt_tokens": response["usage"]["prompt_tokens"],
+                "completion_tokens": response["usage"].get("completion_tokens", 0),
+                "total_tokens": response["usage"]["total_tokens"],
+            })
             cls._history_dict[key] = value
             cls._count_create += 1
             return
@@ -194,7 +223,9 @@ class Completion(openai_Completion):
         cls._count_create += 1
 
     @classmethod
-    def _get_response(cls, config: Dict, raise_on_ratelimit_or_timeout=False, use_cache=True):
+    def _get_response(
+        cls, config: dict[str, Any], raise_on_ratelimit_or_timeout: bool = False, use_cache: bool = True
+    ) -> None:
         """Get the response from the openai api call.
 
         Try cache first. If not found, call the openai api. If the api call fails, retry after retry_wait_time.
@@ -238,12 +269,8 @@ class Completion(openai_Completion):
                 sleep(retry_wait_time)
             except (RateLimitError, Timeout) as err:
                 time_left = max_retry_period - (time.time() - start_time + retry_wait_time)
-                if (
-                    time_left > 0
-                    and isinstance(err, RateLimitError)
-                    or time_left > request_timeout
-                    and isinstance(err, Timeout)
-                    and "request_timeout" not in config
+                if (time_left > 0 and isinstance(err, RateLimitError)) or (
+                    time_left > request_timeout and isinstance(err, Timeout) and "request_timeout" not in config
                 ):
                     if isinstance(err, Timeout):
                         request_timeout <<= 1
@@ -317,7 +344,7 @@ class Completion(openai_Completion):
         return config.copy() if always_copy else config
 
     @classmethod
-    def _get_params_for_create(cls, config: Dict) -> Dict:
+    def _get_params_for_create(cls, config: dict) -> dict:
         """Get the params for the openai api call from a config in the search space."""
         params = cls._pop_subspace(config)
         if cls._prompts:
@@ -334,7 +361,8 @@ class Completion(openai_Completion):
         return params
 
     @classmethod
-    def _eval(cls, config: dict, prune=True, eval_only=False):
+    @require_optional_import("numpy", "flaml")
+    def _eval(cls, config: dict[str, Any], prune: bool = True, eval_only: bool = False):
         """Evaluate the given config as the hyperparameter setting for the openai api call.
 
         Args:
@@ -453,7 +481,7 @@ class Completion(openai_Completion):
                     prune
                     and target_output_tokens
                     and avg_n_tokens <= target_output_tokens * (1 - ratio)
-                    and (num_completions < config_n or num_completions == config_n and data_limit == data_length)
+                    and (num_completions < config_n or (num_completions == config_n and data_limit == data_length))
                 ):
                     # update valid n
                     cls._max_valid_n_per_max_tokens[region_key] = valid_n = cls._max_valid_n_per_max_tokens.get(
@@ -482,7 +510,7 @@ class Completion(openai_Completion):
                                 result[key] += value
                     else:
                         result = metrics
-                for key in result.keys():
+                for key in result:
                     if isinstance(result[key], (float, int)):
                         result[key] /= data_limit
                 result["total_cost"] = cls._total_cost
@@ -506,9 +534,10 @@ class Completion(openai_Completion):
         return result
 
     @classmethod
+    @require_optional_import("flaml", "flaml")
     def tune(
         cls,
-        data: List[Dict],
+        data: list[dict[str, Any]],
         metric: str,
         mode: str,
         eval_func: Callable,
@@ -572,7 +601,7 @@ class Completion(openai_Completion):
             tune.ExperimentAnalysis: The tuning results.
         """
         logger.warning(
-            "tuning via Completion.tune is deprecated in pyautogen v0.2 and openai>=1. "
+            "tuning via Completion.tune is deprecated in autogen, pyautogen v0.2 and openai>=1. "
             "flaml.tune supports tuning more generically."
         )
         if ERROR:
@@ -708,10 +737,10 @@ class Completion(openai_Completion):
     @classmethod
     def create(
         cls,
-        context: Optional[Dict] = None,
+        context: Optional[dict[str, Any]] = None,
         use_cache: Optional[bool] = True,
-        config_list: Optional[List[Dict]] = None,
-        filter_func: Optional[Callable[[Dict, Dict], bool]] = None,
+        config_list: Optional[list[dict[str, Any]]] = None,
+        filter_func: Optional[Callable[[dict[str, Any], dict[str, Any]], bool]] = None,
         raise_on_ratelimit_or_timeout: Optional[bool] = True,
         allow_format_str_template: Optional[bool] = False,
         **config,
@@ -724,47 +753,44 @@ class Completion(openai_Completion):
                 E.g., `prompt="Complete the following sentence: {prefix}, context={"prefix": "Today I feel"}`.
                 The actual prompt will be:
                 "Complete the following sentence: Today I feel".
-                More examples can be found at [templating](https://microsoft.github.io/autogen/docs/Use-Cases/enhanced_inference#templating).
             use_cache (bool, Optional): Whether to use cached responses.
             config_list (List, Optional): List of configurations for the completion to try.
                 The first one that does not raise an error will be used.
                 Only the differences from the default config need to be provided.
                 E.g.,
 
-        ```python
-        response = oai.Completion.create(
-            config_list=[
-                {
-                    "model": "gpt-4",
-                    "api_key": os.environ.get("AZURE_OPENAI_API_KEY"),
-                    "api_type": "azure",
-                    "base_url": os.environ.get("AZURE_OPENAI_API_BASE"),
-                    "api_version": "2024-02-15-preview",
-                },
-                {
-                    "model": "gpt-3.5-turbo",
-                    "api_key": os.environ.get("OPENAI_API_KEY"),
-                    "api_type": "openai",
-                    "base_url": "https://api.openai.com/v1",
-                },
-                {
-                    "model": "llama-7B",
-                    "base_url": "http://127.0.0.1:8080",
-                    "api_type": "openai",
-                }
-            ],
-            prompt="Hi",
-        )
-        ```
+                ```python
+                    response = oai.Completion.create(
+                        config_list = [
+                            {
+                                "model": "gpt-4",
+                                "api_key": os.environ.get("AZURE_OPENAI_API_KEY"),
+                                "api_type": "azure",
+                                "base_url": os.environ.get("AZURE_OPENAI_API_BASE"),
+                                "api_version": "2024-02-01",
+                            },
+                            {
+                                "model": "gpt-3.5-turbo",
+                                "api_key": os.environ.get("OPENAI_API_KEY"),
+                                "base_url": "https://api.openai.com/v1",
+                            },
+                            {
+                                "model": "llama-7B",
+                                "base_url": "http://127.0.0.1:8080",
+                            },
+                        ],
+                        prompt="Hi",
+                    )
+                ```
 
             filter_func (Callable, Optional): A function that takes in the context and the response and returns a boolean to indicate whether the response is valid. E.g.,
 
-        ```python
-        def yes_or_no_filter(context, config, response):
-            return context.get("yes_or_no_choice", False) is False or any(
-                text in ["Yes.", "No."] for text in oai.Completion.extract_text(response)
-            )
-        ```
+                ```python
+                    def yes_or_no_filter(context, config, response):
+                        return context.get("yes_or_no_choice", False) is False or any(
+                            text in ["Yes.", "No."] for text in oai.Completion.extract_text(response)
+                        )
+                ```
 
             raise_on_ratelimit_or_timeout (bool, Optional): Whether to raise RateLimitError or Timeout when all configs fail.
                 When set to False, -1 will be returned when all configs fail.
@@ -784,15 +810,14 @@ class Completion(openai_Completion):
                 - `pass_filter`: whether the response passes the filter function. None if no filter is provided.
         """
         logger.warning(
-            "Completion.create is deprecated in pyautogen v0.2 and openai>=1. "
+            "Completion.create is deprecated in autogen, pyautogen v0.2 and openai>=1. "
             "The new openai requires initiating a client for inference. "
-            "Please refer to https://microsoft.github.io/autogen/docs/Use-Cases/enhanced_inference#api-unification"
         )
         if ERROR:
             raise ERROR
 
         # Warn if a config list was provided but was empty
-        if type(config_list) is list and len(config_list) == 0:
+        if isinstance(config_list, list) and len(config_list) == 0:
             logger.warning(
                 "Completion was provided with a config_list, but the list was empty. Adopting default OpenAI behavior, which reads from the 'model' parameter instead."
             )
@@ -843,7 +868,7 @@ class Completion(openai_Completion):
     def instantiate(
         cls,
         template: Union[str, None],
-        context: Optional[Dict] = None,
+        context: Optional[dict[str, Any]] = None,
         allow_format_str_template: Optional[bool] = False,
     ):
         if not context or template is None:
@@ -860,18 +885,19 @@ class Completion(openai_Completion):
         messages = config.get("messages") if messages is None else messages
         # either "prompt" should be in config (for being compatible with non-chat models)
         # or "messages" should be in config (for tuning chat models only)
-        if prompt is None and (model in cls.chat_models or issubclass(cls, ChatCompletion)):
-            if messages is None:
-                raise ValueError("Either prompt or messages should be in config for chat models.")
+        if prompt is None and (model in cls.chat_models or issubclass(cls, ChatCompletion)) and messages is None:
+            raise ValueError("Either prompt or messages should be in config for chat models.")
         if prompt is None:
             params["messages"] = (
                 [
-                    {
-                        **m,
-                        "content": cls.instantiate(m["content"], context, allow_format_str_template),
-                    }
-                    if m.get("content")
-                    else m
+                    (
+                        {
+                            **m,
+                            "content": cls.instantiate(m["content"], context, allow_format_str_template),
+                        }
+                        if m.get("content")
+                        else m
+                    )
                     for m in messages
                 ]
                 if context
@@ -891,6 +917,7 @@ class Completion(openai_Completion):
         return params
 
     @classmethod
+    @require_optional_import("numpy", "flaml")
     def test(
         cls,
         data,
@@ -933,7 +960,7 @@ class Completion(openai_Completion):
                 An example agg_method in str:
 
         ```python
-        agg_method = 'median'
+        agg_method = "median"
         ```
                 An example agg_method in a Callable:
 
@@ -944,7 +971,7 @@ class Completion(openai_Completion):
                 An example agg_method in a dict of Callable:
 
         ```python
-        agg_method={'median_success': np.median, 'avg_success': np.mean}
+        agg_method = {"median_success": np.median, "avg_success": np.mean}
         ```
 
             return_responses_and_per_instance_result (bool): Whether to also return responses
@@ -979,7 +1006,7 @@ class Completion(openai_Completion):
                 return
             if not metric_keys:
                 metric_keys = []
-                for k in metrics.keys():
+                for k in metrics:
                     try:
                         _ = float(metrics[k])
                         metric_keys.append(k)
@@ -1043,13 +1070,13 @@ class Completion(openai_Completion):
         usage = response["usage"]
         n_input_tokens = usage["prompt_tokens"]
         n_output_tokens = usage.get("completion_tokens", 0)
-        price1K = cls.price1K[model]
+        price1K = cls.price1K[model]  # noqa: N806
         if isinstance(price1K, tuple):
             return (price1K[0] * n_input_tokens + price1K[1] * n_output_tokens) / 1000
         return price1K * (n_input_tokens + n_output_tokens) / 1000
 
     @classmethod
-    def extract_text(cls, response: dict) -> List[str]:
+    def extract_text(cls, response: dict) -> list[str]:
         """Extract the text from a completion or chat response.
 
         Args:
@@ -1064,7 +1091,7 @@ class Completion(openai_Completion):
         return [choice["message"].get("content", "") for choice in choices]
 
     @classmethod
-    def extract_text_or_function_call(cls, response: dict) -> List[str]:
+    def extract_text_or_function_call(cls, response: dict) -> list[str]:
         """Extract the text or function calls from a completion or chat response.
 
         Args:
@@ -1083,12 +1110,12 @@ class Completion(openai_Completion):
 
     @classmethod
     @property
-    def logged_history(cls) -> Dict:
+    def logged_history(cls) -> dict:
         """Return the book keeping dictionary."""
         return cls._history_dict
 
     @classmethod
-    def print_usage_summary(cls) -> Dict:
+    def print_usage_summary(cls) -> dict:
         """Return the usage summary."""
         if cls._history_dict is None:
             print("No usage summary available.", flush=True)
@@ -1127,7 +1154,10 @@ class Completion(openai_Completion):
 
     @classmethod
     def start_logging(
-        cls, history_dict: Optional[Dict] = None, compact: Optional[bool] = True, reset_counter: Optional[bool] = True
+        cls,
+        history_dict: Optional[dict[str, Any]] = None,
+        compact: Optional[bool] = True,
+        reset_counter: Optional[bool] = True,
     ):
         """Start book keeping.
 
@@ -1173,7 +1203,7 @@ class Completion(openai_Completion):
             reset_counter (bool): whether to reset the counter of the number of API calls.
         """
         logger.warning(
-            "logging via Completion.start_logging is deprecated in pyautogen v0.2. "
+            "logging via Completion.start_logging is deprecated in autogen and pyautogen v0.2. "
             "logging via OpenAIWrapper will be added back in a future release."
         )
         if ERROR:
@@ -1188,9 +1218,10 @@ class Completion(openai_Completion):
         cls._history_dict = cls._count_create = None
 
 
+@export_module("autogen")
 class ChatCompletion(Completion):
-    """(openai<1) A class for OpenAI API ChatCompletion. Share the same API as Completion."""
+    """`(openai<1)` A class for OpenAI API ChatCompletion. Share the same API as Completion."""
 
     default_search_space = Completion.default_search_space.copy()
-    default_search_space["model"] = tune.choice(["gpt-3.5-turbo", "gpt-4"])
+    default_search_space["model"] = tune.choice(["gpt-3.5-turbo", "gpt-4"]) if FLAML_INSTALLED else {}
     openai_completion_class = not ERROR and openai.ChatCompletion
