@@ -1,26 +1,82 @@
+# Copyright (c) 2023 - 2025, AG2ai, Inc., AG2ai open-source projects maintainers and core contributors
+#
+# SPDX-License-Identifier: Apache-2.0
+#
+# Portions derived from  https://github.com/microsoft/autogen are under the MIT License.
+# SPDX-License-Identifier: MIT
+
+import importlib
+import importlib.metadata
 import json
 import logging
 import os
 import re
 import tempfile
+import time
+from copy import deepcopy
 from pathlib import Path
-from typing import Any, Dict, List, Optional, Set, Union
+from typing import TYPE_CHECKING, Any, Optional, Union
 
 from dotenv import find_dotenv, load_dotenv
+from packaging.version import parse
 
-try:
+if TYPE_CHECKING:
     from openai import OpenAI
     from openai.types.beta.assistant import Assistant
 
-    ERROR = None
-except ImportError:
-    ERROR = ImportError("Please install openai>=1 to use autogen.OpenAIWrapper.")
-    OpenAI = object
-    Assistant = object
+from ..doc_utils import export_module
 
-NON_CACHE_KEY = ["api_key", "base_url", "api_type", "api_version"]
-DEFAULT_AZURE_API_VERSION = "2024-02-15-preview"
+NON_CACHE_KEY = [
+    "api_key",
+    "base_url",
+    "api_type",
+    "api_version",
+    "azure_ad_token",
+    "azure_ad_token_provider",
+    "credentials",
+]
+DEFAULT_AZURE_API_VERSION = "2024-02-01"
+
+# The below pricing is for 1K tokens. Whenever there is an update in the LLM's pricing,
+# Please convert it to 1K tokens and update in the below dictionary in the format: (input_token_price, output_token_price).
 OAI_PRICE1K = {
+    # https://openai.com/api/pricing/
+    # o1
+    "o1-preview-2024-09-12": (0.0015, 0.0060),
+    "o1-preview": (0.0015, 0.0060),
+    "o1-mini-2024-09-12": (0.0003, 0.0012),
+    "o1-mini": (0.0003, 0.0012),
+    "o1": (0.0015, 0.0060),
+    "o1-2024-12-17": (0.0015, 0.0060),
+    # gpt-4o
+    "gpt-4o": (0.005, 0.015),
+    "gpt-4o-2024-05-13": (0.005, 0.015),
+    "gpt-4o-2024-08-06": (0.0025, 0.01),
+    "gpt-4o-2024-11-20": (0.0025, 0.01),
+    # gpt-4-turbo
+    "gpt-4-turbo-2024-04-09": (0.01, 0.03),
+    # gpt-4
+    "gpt-4": (0.03, 0.06),
+    "gpt-4-32k": (0.06, 0.12),
+    # gpt-4o-mini
+    "gpt-4o-mini": (0.000150, 0.000600),
+    "gpt-4o-mini-2024-07-18": (0.000150, 0.000600),
+    # gpt-3.5 turbo
+    "gpt-3.5-turbo": (0.0005, 0.0015),  # default is 0125
+    "gpt-3.5-turbo-0125": (0.0005, 0.0015),  # 16k
+    "gpt-3.5-turbo-instruct": (0.0015, 0.002),
+    # base model
+    "davinci-002": 0.002,
+    "babbage-002": 0.0004,
+    # old model
+    "gpt-4-0125-preview": (0.01, 0.03),
+    "gpt-4-1106-preview": (0.01, 0.03),
+    "gpt-4-1106-vision-preview": (0.01, 0.03),  # TODO: support vision pricing of images
+    "gpt-3.5-turbo-1106": (0.001, 0.002),
+    "gpt-3.5-turbo-0613": (0.0015, 0.002),
+    # "gpt-3.5-turbo-16k": (0.003, 0.004),
+    "gpt-3.5-turbo-16k-0613": (0.003, 0.004),
+    "gpt-3.5-turbo-0301": (0.0015, 0.002),
     "text-ada-001": 0.0004,
     "text-babbage-001": 0.0005,
     "text-curie-001": 0.002,
@@ -28,32 +84,26 @@ OAI_PRICE1K = {
     "code-davinci-002": 0.1,
     "text-davinci-002": 0.02,
     "text-davinci-003": 0.02,
-    "gpt-3.5-turbo-instruct": (0.0015, 0.002),
-    "gpt-3.5-turbo-0301": (0.0015, 0.002),  # deprecate in Sep
-    "gpt-3.5-turbo-0613": (0.0015, 0.002),
-    "gpt-3.5-turbo-16k": (0.003, 0.004),
-    "gpt-3.5-turbo-16k-0613": (0.003, 0.004),
-    "gpt-35-turbo": (0.0015, 0.002),
-    "gpt-35-turbo-16k": (0.003, 0.004),
-    "gpt-35-turbo-instruct": (0.0015, 0.002),
-    "gpt-4": (0.03, 0.06),
-    "gpt-4-32k": (0.06, 0.12),
     "gpt-4-0314": (0.03, 0.06),  # deprecate in Sep
     "gpt-4-32k-0314": (0.06, 0.12),  # deprecate in Sep
     "gpt-4-0613": (0.03, 0.06),
     "gpt-4-32k-0613": (0.06, 0.12),
-    # 11-06
-    "gpt-3.5-turbo": (0.0015, 0.002),  # default is still 0613
-    "gpt-3.5-turbo-1106": (0.001, 0.002),
-    "gpt-35-turbo-1106": (0.001, 0.002),
-    "gpt-4-1106-preview": (0.01, 0.03),
-    "gpt-4-0125-preview": (0.01, 0.03),
     "gpt-4-turbo-preview": (0.01, 0.03),
-    "gpt-4-1106-vision-preview": (0.01, 0.03),  # TODO: support vision pricing of images
+    # https://azure.microsoft.com/en-us/pricing/details/cognitive-services/openai-service/#pricing
+    "gpt-35-turbo": (0.0005, 0.0015),  # what's the default? using 0125 here.
+    "gpt-35-turbo-0125": (0.0005, 0.0015),
+    "gpt-35-turbo-instruct": (0.0015, 0.002),
+    "gpt-35-turbo-1106": (0.001, 0.002),
+    "gpt-35-turbo-0613": (0.0015, 0.002),
+    "gpt-35-turbo-0301": (0.0015, 0.002),
+    "gpt-35-turbo-16k": (0.003, 0.004),
+    "gpt-35-turbo-16k-0613": (0.003, 0.004),
+    # deepseek
+    "deepseek-chat": (0.00027, 0.0011),
 }
 
 
-def get_key(config: Dict[str, Any]) -> str:
+def get_key(config: dict[str, Any]) -> str:
     """Get a unique identifier of a configuration.
 
     Args:
@@ -75,8 +125,10 @@ def get_key(config: Dict[str, Any]) -> str:
     return json.dumps(config, sort_keys=True)
 
 
-def is_valid_api_key(api_key: str):
-    """Determine if input is valid OpenAI API key.
+def is_valid_api_key(api_key: str) -> bool:
+    """Determine if input is valid OpenAI API key. As of 2024-09-24 there's no official definition of the key structure
+    so we will allow anything starting with "sk-" and having at least 48 alphanumeric (plus underscore and dash) characters.
+    Keys are known to start with "sk-", "sk-proj", "sk-None", and "sk-svcaat"
 
     Args:
         api_key (str): An input string to be validated.
@@ -84,13 +136,17 @@ def is_valid_api_key(api_key: str):
     Returns:
         bool: A boolean that indicates if input is valid OpenAI API key.
     """
-    api_key_re = re.compile(r"^sk-[A-Za-z0-9]{32,}$")
+    api_key_re = re.compile(r"^sk-[A-Za-z0-9_-]{48,}$")
     return bool(re.fullmatch(api_key_re, api_key))
 
 
+@export_module("autogen")
 def get_config_list(
-    api_keys: List, base_urls: Optional[List] = None, api_type: Optional[str] = None, api_version: Optional[str] = None
-) -> List[Dict]:
+    api_keys: list[str],
+    base_urls: Optional[list[str]] = None,
+    api_type: Optional[str] = None,
+    api_version: Optional[str] = None,
+) -> list[dict[str, Any]]:
     """Get a list of configs for OpenAI API client.
 
     Args:
@@ -105,14 +161,14 @@ def get_config_list(
     Example:
     ```python
     # Define a list of API keys
-    api_keys = ['key1', 'key2', 'key3']
+    api_keys = ["key1", "key2", "key3"]
 
     # Optionally, define a list of base URLs corresponding to each API key
-    base_urls = ['https://api.service1.com', 'https://api.service2.com', 'https://api.service3.com']
+    base_urls = ["https://api.service1.com", "https://api.service2.com", "https://api.service3.com"]
 
     # Optionally, define the API type and version if they are common for all keys
-    api_type = 'azure'
-    api_version = '2024-02-15-preview'
+    api_type = "azure"
+    api_version = "2024-02-01"
 
     # Call the get_config_list function to get a list of configuration dictionaries
     config_list = get_config_list(api_keys, base_urls, api_type, api_version)
@@ -136,6 +192,31 @@ def get_config_list(
     return config_list
 
 
+@export_module("autogen")
+def get_first_llm_config(llm_config: dict[str, Any]) -> dict[str, Any]:
+    """Get the first LLM config from the given LLM config.
+
+    Args:
+        llm_config (dict): The LLM config.
+
+    Returns:
+        dict: The first LLM config.
+
+    Raises:
+        ValueError: If the LLM config is invalid.
+    """
+    llm_config = deepcopy(llm_config)
+    if "config_list" not in llm_config:
+        if "model" in llm_config:
+            return llm_config
+        raise ValueError("llm_config must be a valid config dictionary.")
+
+    if len(llm_config["config_list"]) == 0:
+        raise ValueError("Config list must contain at least one config.")
+    return llm_config["config_list"][0]  # type: ignore [no-any-return]
+
+
+@export_module("autogen")
 def config_list_openai_aoai(
     key_file_path: Optional[str] = ".",
     openai_api_key_file: Optional[str] = "key_openai.txt",
@@ -143,7 +224,7 @@ def config_list_openai_aoai(
     openai_api_base_file: Optional[str] = "base_openai.txt",
     aoai_api_base_file: Optional[str] = "base_aoai.txt",
     exclude: Optional[str] = None,
-) -> List[Dict]:
+) -> list[dict[str, Any]]:
     """Get a list of configs for OpenAI API client (including Azure or local model deployments that support OpenAI's chat completion API).
 
     This function constructs configurations by reading API keys and base URLs from environment variables or text files.
@@ -250,8 +331,8 @@ def config_list_openai_aoai(
         else []
     )
     # process openai base urls
-    base_urls = os.environ.get("OPENAI_API_BASE", None)
-    base_urls = base_urls if base_urls is None else base_urls.split("\n")
+    base_urls_env_var = os.environ.get("OPENAI_API_BASE", None)
+    base_urls = base_urls_env_var if base_urls_env_var is None else base_urls_env_var.split("\n")
     openai_config = (
         get_config_list(
             # Assuming OpenAI API_KEY in os.environ["OPENAI_API_KEY"]
@@ -265,16 +346,16 @@ def config_list_openai_aoai(
     return config_list
 
 
+@export_module("autogen")
 def config_list_from_models(
     key_file_path: Optional[str] = ".",
     openai_api_key_file: Optional[str] = "key_openai.txt",
     aoai_api_key_file: Optional[str] = "key_aoai.txt",
     aoai_api_base_file: Optional[str] = "base_aoai.txt",
     exclude: Optional[str] = None,
-    model_list: Optional[list] = None,
-) -> List[Dict]:
-    """
-    Get a list of configs for API calls with models specified in the model list.
+    model_list: Optional[list[str]] = None,
+) -> list[dict[str, Any]]:
+    """Get a list of configs for API calls with models specified in the model list.
 
     This function extends `config_list_openai_aoai` by allowing to clone its' out for each of the models provided.
     Each configuration will have a 'model' key with the model name as its value. This is particularly useful when
@@ -294,15 +375,15 @@ def config_list_from_models(
     Example:
     ```python
     # Define the path where the API key files are located
-    key_file_path = '/path/to/key/files'
+    key_file_path = "/path/to/key/files"
 
     # Define the file names for the OpenAI and Azure OpenAI API keys and bases
-    openai_api_key_file = 'key_openai.txt'
-    aoai_api_key_file = 'key_aoai.txt'
-    aoai_api_base_file = 'base_aoai.txt'
+    openai_api_key_file = "key_openai.txt"
+    aoai_api_key_file = "key_aoai.txt"
+    aoai_api_base_file = "base_aoai.txt"
 
     # Define the list of models for which to create configurations
-    model_list = ['gpt-4', 'gpt-3.5-turbo']
+    model_list = ["gpt-4", "gpt-3.5-turbo"]
 
     # Call the function to get a list of configuration dictionaries
     config_list = config_list_from_models(
@@ -310,7 +391,7 @@ def config_list_from_models(
         openai_api_key_file=openai_api_key_file,
         aoai_api_key_file=aoai_api_key_file,
         aoai_api_base_file=aoai_api_base_file,
-        model_list=model_list
+        model_list=model_list,
     )
 
     # The `config_list` will contain configurations for the specified models, for example:
@@ -332,13 +413,14 @@ def config_list_from_models(
     return config_list
 
 
+@export_module("autogen")
 def config_list_gpt4_gpt35(
     key_file_path: Optional[str] = ".",
     openai_api_key_file: Optional[str] = "key_openai.txt",
     aoai_api_key_file: Optional[str] = "key_aoai.txt",
     aoai_api_base_file: Optional[str] = "base_aoai.txt",
     exclude: Optional[str] = None,
-) -> List[Dict]:
+) -> list[dict[str, Any]]:
     """Get a list of configs for 'gpt-4' followed by 'gpt-3.5-turbo' API calls.
 
     Args:
@@ -361,11 +443,14 @@ def config_list_gpt4_gpt35(
     )
 
 
-def filter_config(config_list, filter_dict):
-    """
-    This function filters `config_list` by checking each configuration dictionary against the
-    criteria specified in `filter_dict`. A configuration dictionary is retained if for every
-    key in `filter_dict`, see example below.
+@export_module("autogen")
+def filter_config(
+    config_list: list[dict[str, Any]],
+    filter_dict: Optional[dict[str, Union[list[Union[str, None]], set[Union[str, None]]]]],
+    exclude: bool = False,
+) -> list[dict[str, Any]]:
+    """This function filters `config_list` by checking each configuration dictionary against the criteria specified in
+    `filter_dict`. A configuration dictionary is retained if for every key in `filter_dict`, see example below.
 
     Args:
         config_list (list of dict): A list of configuration dictionaries to be filtered.
@@ -376,78 +461,75 @@ def filter_config(config_list, filter_dict):
                             when it is found in the list of acceptable values. If the configuration's
                             field's value is a list, then a match occurs if there is a non-empty
                             intersection with the acceptable values.
-
+        exclude (bool): If False (the default value), configs that match the filter will be included in the returned
+            list. If True, configs that match the filter will be excluded in the returned list.
 
     Returns:
         list of dict: A list of configuration dictionaries that meet all the criteria specified
                       in `filter_dict`.
 
     Example:
-    ```python
-    # Example configuration list with various models and API types
-    configs = [
-        {'model': 'gpt-3.5-turbo'},
-        {'model': 'gpt-4'},
-        {'model': 'gpt-3.5-turbo', 'api_type': 'azure'},
-        {'model': 'gpt-3.5-turbo', 'tags': ['gpt35_turbo', 'gpt-35-turbo']},
-    ]
-
-    # Define filter criteria to select configurations for the 'gpt-3.5-turbo' model
-    # that are also using the 'azure' API type
-    filter_criteria = {
-        'model': ['gpt-3.5-turbo'],  # Only accept configurations for 'gpt-3.5-turbo'
-        'api_type': ['azure']       # Only accept configurations for 'azure' API type
-    }
-
-    # Apply the filter to the configuration list
-    filtered_configs = filter_config(configs, filter_criteria)
-
-    # The resulting `filtered_configs` will be:
-    # [{'model': 'gpt-3.5-turbo', 'api_type': 'azure', ...}]
-
-
-    # Define a filter to select a given tag
-    filter_criteria = {
-        'tags': ['gpt35_turbo'],
-    }
-
-    # Apply the filter to the configuration list
-    filtered_configs = filter_config(configs, filter_criteria)
-
-    # The resulting `filtered_configs` will be:
-    # [{'model': 'gpt-3.5-turbo', 'tags': ['gpt35_turbo', 'gpt-35-turbo']}]
-    ```
-
+        ```python
+        # Example configuration list with various models and API types
+        configs = [
+            {"model": "gpt-3.5-turbo"},
+            {"model": "gpt-4"},
+            {"model": "gpt-3.5-turbo", "api_type": "azure"},
+            {"model": "gpt-3.5-turbo", "tags": ["gpt35_turbo", "gpt-35-turbo"]},
+        ]
+        # Define filter criteria to select configurations for the 'gpt-3.5-turbo' model
+        # that are also using the 'azure' API type
+        filter_criteria = {
+            "model": ["gpt-3.5-turbo"],  # Only accept configurations for 'gpt-3.5-turbo'
+            "api_type": ["azure"],  # Only accept configurations for 'azure' API type
+        }
+        # Apply the filter to the configuration list
+        filtered_configs = filter_config(configs, filter_criteria)
+        # The resulting `filtered_configs` will be:
+        # [{'model': 'gpt-3.5-turbo', 'api_type': 'azure', ...}]
+        # Define a filter to select a given tag
+        filter_criteria = {
+            "tags": ["gpt35_turbo"],
+        }
+        # Apply the filter to the configuration list
+        filtered_configs = filter_config(configs, filter_criteria)
+        # The resulting `filtered_configs` will be:
+        # [{'model': 'gpt-3.5-turbo', 'tags': ['gpt35_turbo', 'gpt-35-turbo']}]
+        ```
     Note:
         - If `filter_dict` is empty or None, no filtering is applied and `config_list` is returned as is.
         - If a configuration dictionary in `config_list` does not contain a key specified in `filter_dict`,
           it is considered a non-match and is excluded from the result.
         - If the list of acceptable values for a key in `filter_dict` includes None, then configuration
           dictionaries that do not have that key will also be considered a match.
+
     """
-
-    def _satisfies(config_value, acceptable_values):
-        if isinstance(config_value, list):
-            return bool(set(config_value) & set(acceptable_values))  # Non-empty intersection
-        else:
-            return config_value in acceptable_values
-
     if filter_dict:
-        config_list = [
-            config
-            for config in config_list
-            if all(_satisfies(config.get(key), value) for key, value in filter_dict.items())
+        return [
+            item
+            for item in config_list
+            if all(_satisfies_criteria(item.get(key), values) != exclude for key, values in filter_dict.items())
         ]
     return config_list
 
 
+def _satisfies_criteria(value: Any, criteria_values: Any) -> bool:
+    if value is None:
+        return False
+
+    if isinstance(value, list):
+        return bool(set(value) & set(criteria_values))  # Non-empty intersection
+    else:
+        return value in criteria_values
+
+
+@export_module("autogen")
 def config_list_from_json(
     env_or_file: str,
     file_location: Optional[str] = "",
-    filter_dict: Optional[Dict[str, Union[List[Union[str, None]], Set[Union[str, None]]]]] = None,
-) -> List[Dict]:
-    """
-    Retrieves a list of API configurations from a JSON stored in an environment variable or a file.
+    filter_dict: Optional[dict[str, Union[list[Union[str, None]], set[Union[str, None]]]]] = None,
+) -> list[dict[str, Any]]:
+    """Retrieves a list of API configurations from a JSON stored in an environment variable or a file.
 
     This function attempts to parse JSON data from the given `env_or_file` parameter. If `env_or_file` is an
     environment variable containing JSON data, it will be used directly. Otherwise, it is assumed to be a filename,
@@ -472,7 +554,7 @@ def config_list_from_json(
 
     # We can retrieve a filtered list of configurations like this:
     filter_criteria = {"model": ["gpt-3.5-turbo"]}
-    configs = config_list_from_json('CONFIG_JSON', filter_dict=filter_criteria)
+    configs = config_list_from_json("CONFIG_JSON", filter_dict=filter_criteria)
     # The 'configs' variable will now contain only the configurations that match the filter criteria.
     ```
 
@@ -488,7 +570,7 @@ def config_list_from_json(
         # The environment variable exists. We should use information from it.
         if os.path.exists(env_str):
             # It is a file location, and we need to load the json from the file.
-            with open(env_str, "r") as file:
+            with open(env_str) as file:
                 json_str = file.read()
         else:
             # Else, it should be a JSON string by itself.
@@ -497,25 +579,27 @@ def config_list_from_json(
     else:
         # The environment variable does not exist.
         # So, `env_or_file` is a filename. We should use the file location.
-        config_list_path = os.path.join(file_location, env_or_file)
+        config_list_path = os.path.join(file_location, env_or_file) if file_location is not None else env_or_file
+
         with open(config_list_path) as json_file:
             config_list = json.load(json_file)
+
+    config_list = filter_config(config_list, filter_dict)
+
     return filter_config(config_list, filter_dict)
 
 
 def get_config(
-    api_key: str, base_url: Optional[str] = None, api_type: Optional[str] = None, api_version: Optional[str] = None
-) -> Dict:
-    """
-    Constructs a configuration dictionary for a single model with the provided API configurations.
+    api_key: Optional[str],
+    base_url: Optional[str] = None,
+    api_type: Optional[str] = None,
+    api_version: Optional[str] = None,
+) -> dict[str, Any]:
+    """Constructs a configuration dictionary for a single model with the provided API configurations.
 
     Example:
     ```python
-    config = get_config(
-        api_key="sk-abcdef1234567890",
-        base_url="https://api.openai.com",
-        api_version="v1"
-    )
+    config = get_config(api_key="sk-abcdef1234567890", base_url="https://api.openai.com", api_version="v1")
     # The 'config' variable will now contain:
     # {
     #     "api_key": "sk-abcdef1234567890",
@@ -535,19 +619,21 @@ def get_config(
     """
     config = {"api_key": api_key}
     if base_url:
-        config["base_url"] = base_url
+        config["base_url"] = os.getenv(base_url, default=base_url)
     if api_type:
-        config["api_type"] = api_type
+        config["api_type"] = os.getenv(api_type, default=api_type)
     if api_version:
-        config["api_version"] = api_version
+        config["api_version"] = os.getenv(api_version, default=api_version)
     return config
 
 
+@export_module("autogen")
 def config_list_from_dotenv(
-    dotenv_file_path: Optional[str] = None, model_api_key_map: Optional[dict] = None, filter_dict: Optional[dict] = None
-) -> List[Dict[str, Union[str, Set[str]]]]:
-    """
-    Load API configurations from a specified .env file or environment variables and construct a list of configurations.
+    dotenv_file_path: Optional[str] = None,
+    model_api_key_map: Optional[dict[str, Any]] = None,
+    filter_dict: Optional[dict[str, Union[list[Union[str, None]], set[Union[str, None]]]]] = None,
+) -> list[dict[str, Union[str, set[str]]]]:
+    """Load API configurations from a specified .env file or environment variables and construct a list of configurations.
 
     This function will:
     - Load API keys from a provided .env file or from existing environment variables.
@@ -582,9 +668,10 @@ def config_list_from_dotenv(
         else:
             logging.warning(f"The specified .env file {dotenv_path} does not exist.")
     else:
-        dotenv_path = find_dotenv()
-        if not dotenv_path:
+        dotenv_path_str = find_dotenv()
+        if not dotenv_path_str:
             logging.warning("No .env file found. Loading configurations from environment variables.")
+        dotenv_path = Path(dotenv_path_str)
         load_dotenv(dotenv_path)
 
     # Ensure the model_api_key_map is not None to prevent TypeErrors during key assignment.
@@ -643,15 +730,116 @@ def config_list_from_dotenv(
     return config_list
 
 
-def retrieve_assistants_by_name(client: OpenAI, name: str) -> List[Assistant]:
-    """
-    Return the assistants with the given name from OAI assistant API
-    """
-    if ERROR:
-        raise ERROR
+def retrieve_assistants_by_name(client: "OpenAI", name: str) -> list["Assistant"]:
+    """Return the assistants with the given name from OAI assistant API"""
     assistants = client.beta.assistants.list()
     candidate_assistants = []
     for assistant in assistants.data:
         if assistant.name == name:
             candidate_assistants.append(assistant)
     return candidate_assistants
+
+
+def detect_gpt_assistant_api_version() -> str:
+    """Detect the openai assistant API version"""
+    oai_version = importlib.metadata.version("openai")
+    return "v1" if parse(oai_version) < parse("1.21") else "v2"
+
+
+def create_gpt_vector_store(client: "OpenAI", name: str, fild_ids: list[str]) -> Any:
+    """Create a openai vector store for gpt assistant"""
+    try:
+        vector_store = client.beta.vector_stores.create(name=name)
+    except Exception as e:
+        raise AttributeError(f"Failed to create vector store, please install the latest OpenAI python package: {e}")
+
+    # poll the status of the file batch for completion.
+    batch = client.beta.vector_stores.file_batches.create_and_poll(vector_store_id=vector_store.id, file_ids=fild_ids)
+
+    if batch.status == "in_progress":
+        time.sleep(1)
+        logging.debug(f"file batch status: {batch.file_counts}")
+        batch = client.beta.vector_stores.file_batches.poll(vector_store_id=vector_store.id, batch_id=batch.id)
+
+    if batch.status == "completed":
+        return vector_store
+
+    raise ValueError(f"Failed to upload files to vector store {vector_store.id}:{batch.status}")
+
+
+def create_gpt_assistant(
+    client: "OpenAI", name: str, instructions: str, model: str, assistant_config: dict[str, Any]
+) -> "Assistant":
+    """Create a openai gpt assistant"""
+    assistant_create_kwargs = {}
+    gpt_assistant_api_version = detect_gpt_assistant_api_version()
+    tools = assistant_config.get("tools", [])
+
+    if gpt_assistant_api_version == "v2":
+        tool_resources = assistant_config.get("tool_resources", {})
+        file_ids = assistant_config.get("file_ids")
+        if tool_resources.get("file_search") is not None and file_ids is not None:
+            raise ValueError(
+                "Cannot specify both `tool_resources['file_search']` tool and `file_ids` in the assistant config."
+            )
+
+        # Designed for backwards compatibility for the V1 API
+        # Instead of V1 AssistantFile, files are attached to Assistants using the tool_resources object.
+        for tool in tools:
+            if tool["type"] == "retrieval":
+                tool["type"] = "file_search"
+                if file_ids is not None:
+                    # create a vector store for the file search tool
+                    vs = create_gpt_vector_store(client, f"{name}-vectorestore", file_ids)
+                    tool_resources["file_search"] = {
+                        "vector_store_ids": [vs.id],
+                    }
+            elif tool["type"] == "code_interpreter" and file_ids is not None:
+                tool_resources["code_interpreter"] = {
+                    "file_ids": file_ids,
+                }
+
+        assistant_create_kwargs["tools"] = tools
+        if len(tool_resources) > 0:
+            assistant_create_kwargs["tool_resources"] = tool_resources
+    else:
+        # not support forwards compatibility
+        if "tool_resources" in assistant_config:
+            raise ValueError("`tool_resources` argument are not supported in the openai assistant V1 API.")
+        if any(tool["type"] == "file_search" for tool in tools):
+            raise ValueError(
+                "`file_search` tool are not supported in the openai assistant V1 API, please use `retrieval`."
+            )
+        assistant_create_kwargs["tools"] = tools
+        assistant_create_kwargs["file_ids"] = assistant_config.get("file_ids", [])
+
+    logging.info(f"Creating assistant with config: {assistant_create_kwargs}")
+    return client.beta.assistants.create(name=name, instructions=instructions, model=model, **assistant_create_kwargs)
+
+
+def update_gpt_assistant(client: "OpenAI", assistant_id: str, assistant_config: dict[str, Any]) -> "Assistant":
+    """Update openai gpt assistant"""
+    gpt_assistant_api_version = detect_gpt_assistant_api_version()
+    assistant_update_kwargs = {}
+
+    if assistant_config.get("tools") is not None:
+        assistant_update_kwargs["tools"] = assistant_config["tools"]
+
+    if assistant_config.get("instructions") is not None:
+        assistant_update_kwargs["instructions"] = assistant_config["instructions"]
+
+    if gpt_assistant_api_version == "v2":
+        if assistant_config.get("tool_resources") is not None:
+            assistant_update_kwargs["tool_resources"] = assistant_config["tool_resources"]
+    else:
+        if assistant_config.get("file_ids") is not None:
+            assistant_update_kwargs["file_ids"] = assistant_config["file_ids"]
+
+    return client.beta.assistants.update(assistant_id=assistant_id, **assistant_update_kwargs)
+
+
+def _satisfies(config_value: Any, acceptable_values: Any) -> bool:
+    if isinstance(config_value, list):
+        return bool(set(config_value) & set(acceptable_values))  # Non-empty intersection
+    else:
+        return config_value in acceptable_values
